/************************************************************************
RWhois Software

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and
that no charge is made for such copies.

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By:
  Mark Kosters
***************************************************************************/
#include <stdio.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "defines.h"
#include "btree.h"

#define DEFAULT_DB_NAME    "default.db"
#define DEFAULT_INDEX_NAME "default.ndx"
#define USAGE_STR "%s: <-d dbase_name> \n"

/************
  local prototypes
*************/
static FILE *open_file();

/*************************************************************************
  main routine
**************************************************************************/
main(argc, argv)
int argc;
char *argv[];

{
extern int optind;
extern char *optarg;
char c;
int index_no;
FILE *dfp;
#if BTREE_DEF
BTREE *bt;
#else
FILE *bt;
#endif
char dbase_name[MAX_FILENAME];
char index_name[MAX_FILENAME];
char rwhois_area[MAX_FILENAME];

*dbase_name = '\0';
*index_name = '\0';
*rwhois_area = '\0';

setbuf (stdout, NULL);
while ((c = getopt(argc, argv, "d:n:")) != EOF)
  {
  switch (c)
    {
    case 'd':
      strcpy (rwhois_area, optarg);
      break;
    case 'n':
      sprintf (dbase_name, "%s.db", optarg);
      sprintf (index_name, "%s.ndx", optarg);
      break;
    }
  }
if (argc == 1)
  {
  fprintf (stderr, USAGE_STR, argv[0]);
  exit (1);
  }

if (strlen (dbase_name) == 0) {
  strcpy (dbase_name, DEFAULT_DB_NAME);
  strcpy (index_name, DEFAULT_INDEX_NAME);
  }

if ((dfp = open_file (dbase_name, "w")) == NULL || 
#if BTREE_DEF
  (bt = btopen (index_name, O_CREAT|O_RDWR, 0644))== NULL) {
  btperror ("error opening ndx and db files");
#else
  (bt = open_file (index_name, "w"))== NULL) {
  fprintf (stderr, "error opening ndx and db files\n");
#endif
  exit(1);
  }

#if NOK
set_rwhois_index(rwhois_area);
#endif

while (optind<argc) {
  index_no = save_in_db (dfp, argv[optind]);
  index_file (bt, argv[optind], index_no);
  optind++;
  } 
fclose (dfp);
#if BTREE_DEF
btclose (bt);
#else
fclose (bt);
#endif
}

/*************************************************************************
 opens the files
**************************************************************************/
static FILE *open_file (db_file, mode)
char *db_file;
char *mode;

{
FILE *fp;

if((fp = fopen(db_file, mode)) == (FILE *) NULL){
#if NOK
  print_error ("%s", object->attribute_fname);
#endif
  return(NULL);
  }
return (fp);
}

/*************************************************************************
 saves file info into db
**************************************************************************/
int save_in_db (fp, data_file)
FILE *fp;
char *data_file;

{
static int index_no=0;
struct stat file_info;

stat(data_file, &file_info);
if (index_no > 0)
  fprintf (fp, "---\n");
fprintf (fp, "file: %s\n", data_file);
fprintf (fp, "index_no: %d\n", index_no);
fprintf (fp, "size: %d\n", file_info.st_size);
return (index_no++);
}

/*************************************************************************
 indexes file - places in format
   offset:fileno:data
**************************************************************************/
int index_file (bt, data_file, index_no)
#if BTREE_DEF
BTREE *bt;
#else
FILE *bt;
#endif
char *data_file;
int index_no;

{
char line[MAX_LINE];
long offset=0;
FILE *fp;

if ((fp = open_file (data_file, "r")) == NULL)
  return (FALSE);

while (fgets (line, MAX_LINE, fp) != NULL) {
  /* truncation here */
#if BTREE_DEF
  line[BT_KSIZ-1] = '\0';
  if (btinsert (line, index_no,  offset, bt) < 0)
     btperror ("insert failed");
#else
  fprintf (bt, "%l:%d:%s\n", offset, data_file, line);
#endif
  if (strncmp (line, "---", 3) == 0)
    offset = ftell (fp);
  }
fclose (fp);
}

