/************************************************************************
RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and 
that no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By: 
  Mark Kosters
***************************************************************************/
#include <stdio.h>
#include <string.h>
#include <sys/stdtypes.h>
#include <sys/stat.h>
#ifdef POSIX
#include <stdlib.h>
#else
#include <malloc.h>
#endif
#include "defines.h"
#include "misc.h"
#include "client_msgs.h"
#include "llist.h"
#include "attributes.h"
#include "objects.h"


/* Globals */
static llist_struct *global_attributes = NULL;
static llist_struct *internal_attribute_list = NULL;

/****
  local prototypes
****/

static a_index_type select_word_index_type();
static a_type select_type();

/****************************************************************************
  reads the attributes for the class
  returns TRUE if ok
          FALSE if not
****************************************************************************/
int read_attributes (object)
  object_struct *object;
{
  FILE *fp;
  attribute_struct *top = NULL, *cur, *new;
  int lineno;
  char line[BUFSIZ];
  char tag[MAX_TEMPLATE_NAME];
  char datum[MAX_TEMPLATE_NAME];
  char desc[MAX_TEMPLATE_DESC];
  struct stat buf;

  /** this may be for the global type **/
  if (object->attribute_fname == NULL)
    return (TRUE);

  if((fp = fopen(object->attribute_fname, "r")) == (FILE *) NULL){
    print_error (NO_OBJECT_SCHEMA_FOUND, "");
    return(FALSE);
  }

  /* create the version nubmer */
  if( fstat( fileno(fp),  &buf) < 0 ){
    print_error( MISC_DIAG, "Unable to stat schema file '%s'",
                 object->attribute_fname);
    object->version = 0;
  } else 
    object->version = buf.st_mtime;   

  while ((fgets (line, BUFSIZ, fp)) != NULL) {
    stripchar (line, '\n');
    if (new_record (line) || top == NULL) {
      new = NEW_ATTRIBUTE;
      if (top == NULL)
        top = cur = new;
      else {
        cur->object_name = NEW_STRING (object->name);
        cur->global_id = set_attribute_global_id (cur, object);
        cur->internal_id = set_internal_id (cur);
        cur->object_references = append_llist (NULL, object);
        cur->no_object_references = 1;
        cur->nxt = new;
        cur = new;
      }
    }
    if (parse_line (line, tag, datum)) {
      if (strcasecmp (tag, A_ATTRIBUTE) == 0)
        cur->name = NEW_STRING (datum);
      else if (strcasecmp (tag, A_ATTRIB_ALIAS) == 0)
        cur->name_alias = NEW_STRING (datum);
      else if (strcasecmp (tag, A_DESCRIPTION) == 0)
        cur->description = NEW_STRING (datum);
      else if (strcasecmp (tag, A_DEF_OBJECT) == 0)
        cur->def_object = true_false (datum);
      else if (strcasecmp (tag, A_IS_PRIMARY_KEY) == 0)
        cur->primary_key = true_false (datum);
      else if (strcasecmp (tag, A_IS_AUTH_AREA) == 0)
        cur->authority_area = true_false (datum);
      else if (strcasecmp (tag, A_IS_SOA_SERIAL) == 0)
        cur->soa = true_false (datum);
      else if (strcasecmp (tag, A_IS_REQUIRED) == 0)
        cur->required = true_false (datum);
      else if (strcasecmp (tag, A_MULT_LINES) == 0)
        cur->mult_lines = true_false (datum);
      else if (strcasecmp (tag, A_MULTIPLE_GRP) == 0)
        cur->multiple_grp = true_false (datum);
      else if (strcasecmp (tag, A_FORMAT) == 0)
        cur->format = NEW_STRING (datum);
      else if (strcasecmp (tag, A_INDEX) == 0)
        cur->index_words = select_word_index_type (datum);
      else if (strcasecmp (tag, A_TYPE) == 0)
        cur->type = select_type (datum);
    }
  }
  cur->object_name = NEW_STRING (object->name);
  cur->global_id = set_attribute_global_id (new, object);
  cur->internal_id = set_internal_id (cur);
  cur->object_references = append_llist (NULL, object);
  cur->no_object_references = 1;
  fclose(fp);
  object->attributes = top;
#if DEBUG
  fprintf (stderr, "read_attributes: ** begin for |%s|\n", object->name);
  debug_print_attributes (top, TRUE);
  fprintf (stderr, "read_attributes: ** end for |%s|\n", object->name);
#endif
  return(TRUE);
}

/****************************************************************************
  returns the unique number for that particular index
****************************************************************************/
set_attribute_global_id (attrib, object)
  attribute_struct *attrib;
  object_struct *object;

{
  llist_struct *cur;
  attribute_struct *ptr; 
  int i=0;

  cur = global_attributes;
  while (cur != NULL) {
    ptr = cur->datum;
    if (strcasecmp (ptr->name, attrib->name) == 0) {
      ptr->no_object_references++;
      ptr->object_references = append_llist (ptr->object_references, 
                                             object);
      return (i);
    }
    cur = cur->nxt;
    i++;
  }
  global_attributes = append_llist (global_attributes, attrib);
  return (i);
}

/****************************************************************************
  sets a linked list of all the attributes
  returns the uniq internal id
****************************************************************************/
set_internal_id (attrib)
  attribute_struct *attrib;

{
  llist_struct *cur = internal_attribute_list;
  int id = 0;

  if (cur == NULL) {
    internal_attribute_list = append_llist (NULL, attrib);
    return (id);
  }
  id = 1;
  while (cur->nxt != NULL) {
    cur = cur->nxt;
    id++;
  }
  internal_attribute_list = append_llist (internal_attribute_list, attrib);
  return (id);
}

/****************************************************************************
  returns how to index the word based on the attribute
  if none declared, returns all words.
****************************************************************************/
static a_index_type select_word_index_type (itype)
  char *itype;

{
  if (strcasecmp (itype, A_INDEX_NONE_WORDS) == 0)
    return (INDEX_NO_WORDS);
  else if (strcasecmp (itype, A_INDEX_FIRST_WORDS) == 0)
    return (INDEX_FIRST_WORD);
  else if (strcasecmp (itype, A_INDEX_ONLY_WORDS) == 0)
    return (INDEX_WORD_ONLY);
  else 
    return (INDEX_ALL_WORDS);
}

/****************************************************************************
  returns the description of the attribute
****************************************************************************/
static a_type select_type(type)
  char *type;

{
  if (strcasecmp (type, A_SEE_ALSO) == 0)
    return (TYPE_SEE_ALSO);
  else if (strcasecmp (type, A_REFERRAL) == 0)
    return (TYPE_REFERRAL);
  else if (strcasecmp (type, A_TYPE_FILE) == 0)
    return (TYPE_FILE);
  else if (strcasecmp (type, A_TYPE_MIME) == 0)
    return (TYPE_MIME);
  else
    return (TYPE_TEXT);
}


/****************************************************************************
  returns the description of the attribute
****************************************************************************/
char *show_attribute_type(type)
  a_type type;

{
  switch (type) {
  case TYPE_SEE_ALSO:
    return (A_SEE_ALSO);
    break;
  case TYPE_REFERRAL:
    return (A_REFERRAL);
    break;
  case TYPE_FILE:
    return (A_TYPE_FILE);
    break;
  case TYPE_MIME:
    return (A_TYPE_MIME);
    break;
  case TYPE_TEXT:
    return (A_TYPE_TEXT);
    break;
  default:
    return ("UNKNOWN");
    break;
  }
}

/****************************************************************************
  returns the global id of the attribute based on txt 
          -1 if not found
****************************************************************************/
int get_attribute_global_id_from_txt (attribute_txt)
  char *attribute_txt;

{
  llist_struct *cur = global_attributes;
  attribute_struct *ptr;
  int i=0;

  while (cur != NULL) {
    ptr = cur->datum;
    if (strcasecmp (attribute_txt, ptr->name) == 0 || 
        strcasecmp (attribute_txt, ptr->name_alias) == 0) 
      return (i);
    i++;
    cur = cur->nxt;
  }
  return (-1);
}
 
/****************************************************************************
  returns the attribute info for the particular global_id
          NULL if not found
****************************************************************************/
attribute_struct *get_attribute_info_from_global_id (offset)
  int offset;

{
  llist_struct *cur = global_attributes;
  int i=0;

  while (cur != NULL) {
    if (i == offset) 
      return (cur->datum);
    i++;
    cur = cur->nxt;
  }
  return (NULL);
}

/****************************************************************************
  returns the attribute info for the particular global_id
          NULL if not found
****************************************************************************/
attribute_struct *get_attribute_info_from_internal_id (offset)
  int offset;

{
  llist_struct *cur = internal_attribute_list;
  int i=0;

  while (cur != NULL) {
    if (i == offset)
      return (cur->datum);
    i++;
    cur = cur->nxt;
  }
  return (NULL);
}

/****************************************************************************
  returns the attribute info for the particular attribute and object names
****************************************************************************/
attribute_struct *get_attribute_info_from_name_and_object (object_name,
                                                           attribute_name)
  char *object_name;
  char *attribute_name;

{
  attribute_struct *ptr;

  if ((ptr = get_object_attributes_by_object_name (object_name)) != NULL) {
    while (ptr != NULL) {
      if (strcasecmp (ptr->name, attribute_name) == 0 || 
          (ptr->name_alias != NULL && 
           strcasecmp (ptr->name_alias, attribute_name) == 0)) {
        return (ptr);
      }
      ptr = ptr->nxt;
    }
  }
  return (NULL);
}

/****************************************************************************
  returns the global id of the attribute based on txt
          -1 if not found
****************************************************************************/
int get_id_from_object_and_name (object_name, attribute_name)
  char *object_name;
  char *attribute_name;

{
  attribute_struct *ptr;
  int i=0;

  if ((ptr = get_object_attributes_by_object_name (object_name)) != NULL ) {
    while (ptr != NULL) {
      if (strcasecmp (attribute_name, ptr->name) == 0 ||
          (ptr->name_alias != NULL &&
           strcasecmp (attribute_name, ptr->name_alias) == 0))
        return (ptr->internal_id);
      ptr = ptr->nxt;
    }
  }
  return (-1);
}

/****************************************************************************
  returns the name of the attribute
****************************************************************************/
char *get_attribute_name(id)
  int id;

{
  attribute_struct *attrib;
  if (id == -1 || (attrib = get_attribute_info_from_internal_id (id)) == NULL)
    return ("");
  return (attrib->name);
}

/****************************************************************************
  returns the global list of attributes
****************************************************************************/
llist_struct *global_list_of_attributes()
{
  return (global_attributes);
}

/****************************************************************************
  sets the attribute list based on object
****************************************************************************/
attribute_struct *set_attributes_list_from_object (mode, tag, datum)
  int mode;
  char *tag, *datum;

{
  static attribute_struct *cur = NULL;

  /***
    resets the attribute list
    ****/

  if (mode == FALSE) {
    cur = NULL;
    return (NULL);
  }

  /****
    if cur is null, finds the attribute list - else returns cur
    ****/

  printf ("set_attributes_list_from_object: tag |%s| datum |%s|\n", tag,
          datum);

  if (cur == NULL)
    return (cur=get_object_attributes_by_tag (tag, datum));
  else
    return (cur);
}

/****************************************************************************
  finds the specific attribute
****************************************************************************/
attribute_struct *find_specific_attribute (attrib, tag)
  attribute_struct *attrib;
char *tag;

{
  while (attrib != NULL) {
    if(strcasecmp(tag, attrib->name) == 0 ||
       strcasecmp (tag, attrib->name_alias) == 0)
      return (attrib);
    attrib=attrib->nxt;
  }
  return (NULL);
}

/****************************************************************************
  debug stuff
****************************************************************************/
debug_global_llist()
{
  llist_struct *c;

  c = global_list_of_attributes();
  while (c!=NULL)
  {
    debug_print_attributes (c->datum, FALSE);
    c=c->nxt;
  }
}

/****************************************************************************
  debugs the templates
****************************************************************************/
debug_print_attributes(cur, loop)
  attribute_struct *cur;
  int loop;

{
  while (cur != NULL)
  {
    fprintf (stderr, "global_id: |%d|\n", cur ->global_id);
    fprintf (stderr, "internal_id: |%d|\n", cur ->internal_id);
    fprintf (stderr, "primary_key: |%d|\n", cur ->primary_key);
    fprintf (stderr, "required: |%d|\n", cur ->required);
    fprintf (stderr, "mult_lines: |%d|\n", cur ->mult_lines);
    fprintf (stderr, "name: |%s|\n", cur ->name);
    fprintf (stderr, "name_alias: |%s|\n", cur->name_alias);
    fprintf (stderr, "description: |%s|\n", cur->description);
    fprintf (stderr, "object_name: |%s|\n", cur->object_name);
    fprintf (stderr, "no_object_references: |%d|\n",
             cur->no_object_references);
    if (cur->format != NULL)
      fprintf (stderr, "format: |%s|\n", cur->format);
    fprintf (stderr, "index-type: |%d|\n", cur->index_words);
    fprintf (stderr, "authority_area: |%d|\n", cur->authority_area);
    fprintf (stderr, "---\n");
    if (!loop)
      return;
    cur = cur->nxt;
  }
  fprintf (stderr, "debug_print_attributes: done!\n");
}
