/************************************************************************
RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and 
that no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
					scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
					markk@internic.net


Developed By: 
  Mark Kosters
***************************************************************************/
#include <stdio.h>
#include <string.h>
#include "defines.h"
#include "misc.h"
#include "records.h"
#include "objects.h"
#include "commands.h"
#include "output.h"
#include "client_msgs.h"
#include "whois.h"
#include "forward.h"
#include "read_config.h"

/******
  local globals
******/

static output_command_struct *output_command_head = NULL;
static char *default_multi = NULL; 
static char *default_single = NULL; 

/*******
  local prototypes
********/

static int find_command_len();
static output_format_struct *retrieve_output_format();
static int display_formatted_results();
static void display_record();
static llist_struct *display_field();
void display_text();
static char *display_char();
static void display_conditional_text();
static int display_file();
static int display_mime();
static void do_see_also();
static int run_sub_query();
static void display_referral();
static llist_struct *setup_deferred_subqueries();
static void read_header_and_footers();
static int required_fields_in_record();
static void display_attribute();

/****************************************************************************
  reads the main configuration file
  returns TRUE if ok
          FALSE otherwise
****************************************************************************/
int read_displays(config_file)
char *config_file;

{
FILE *fp;
char line[BUFSIZ];
char tag[MAX_TEMPLATE_DESC];
char datum[MAX_TEMPLATE_DESC];
int retval = TRUE;
output_command_struct *cur, *new;

if((fp = fopen(config_file, "r")) == (FILE *) NULL){
  print_error (MISC_DIAG, "Can not open displays file '%s'", config_file);
  return(FALSE);
  }

while ((fgets (line, BUFSIZ, fp)) != NULL) {
  if (new_record (line) || output_command_head == NULL) {
    if((new = NEW_OUTPUT_COMMAND) == (output_command_struct *)NULL) {
      print_error (MEMORY_ALLOCATION_PROBLEM, "");
      return(FALSE);
      }
    if (output_command_head == NULL)
      cur = output_command_head = new;
    else {
      cur->nxt = new;
      cur = new;
      }
    }
  if (parse_line (line, tag, datum)) {
    if (strcasecmp (tag, O_COMMAND) == 0)
      cur->command = NEW_STRING (datum);
    else if (strcasecmp (tag, O_COMMAND_LEN) == 0)
      cur->command_len = atoi (datum);
    else if (strcasecmp (tag, O_MAGIC) == 0)
      cur->magic = NEW_STRING (datum);
    else if (strcasecmp (tag, O_DESCRIPTION) == 0)
      cur->description = NEW_STRING (datum);
    else if (strcasecmp (tag, O_HEADER) == 0)
      cur->header = NEW_STRING (datum);
    else if (strcasecmp (tag, O_FOOTER) == 0)
      cur->footer = NEW_STRING (datum);
    else if (strcasecmp (tag, O_RECORD_DELIM) == 0)
      cur->record_delim = NEW_STRING (datum);
    else if (strcasecmp (tag, O_DEFAULT_MULTI) == 0)
      default_multi = NEW_STRING (datum);
    else if (strcasecmp (tag, O_DEFAULT_SINGLE) == 0)
      default_single = NEW_STRING (datum);
    }
  }
/******
  hardcode dump command
******/

if((new = NEW_OUTPUT_COMMAND) == (output_command_struct *)NULL) {
  print_error (MEMORY_ALLOCATION_PROBLEM, "");
  return(FALSE);
  }
if (output_command_head == NULL)
  cur = output_command_head = new;
else {
  cur->nxt = new;
  cur = new;
  }
cur->command = NEW_STRING ("dump");
cur->command_len = 4;
cur->magic = NEW_STRING ("#");
cur->description = NEW_STRING ("display in tag/value format");

cur = output_command_head;
while (cur != NULL) {
  if (cur->command != NULL)
    append_command (cur->command, cur->command_len, cur->magic, NULL, TRUE, 
      COMMAND_DISPLAY, NULL, NULL, NULL);
  cur=cur->nxt;
  }
return (TRUE);
}

/****************************************************************************
  reads the display command, etc from the template
  expected format:
    command:length:filename
****************************************************************************/
output_display_struct *read_display_format_from_object(top, str)
output_display_struct *top;
char *str;

{
output_display_struct *new, *cur;
char command[MAX_LINE];
char command_len[MAX_LINE];
char filename[MAX_LINE];

if (parse_line (str, command, filename)) {
  new = NEW_OUTPUT_DISPLAY;
  new->command = NEW_STRING (command);
  strcpy (str, filename);
  new->command_len = find_command_len (command);
  new->filename = NEW_STRING (str);
  new->format = encode_output_format (new->format, new->filename);
  if (top == NULL)
    return (new);
  else {
    cur = top;
    while (cur->nxt != NULL)
      cur = cur->nxt;
    cur->nxt = new;
    }
  }
return (top);
}

/****************************************************************************
  return_all_defined_displays
****************************************************************************/
output_command_struct *return_all_defined_displays()
{
return (output_command_head);
}

/****************************************************************************
  finds the command from the config file
  returns len if ok
          <0 otherwise
****************************************************************************/
static int find_command_len (command)
char *command;

{
output_command_struct *cur = output_command_head;

while (cur != NULL) {
  if (cur->command != NULL && strcasecmp (cur->command, command)==0) {
    if (cur->command_len)
      return (cur->command_len);
    else
      return (strlen (command));
    }
  cur=cur->nxt;
  }
return (-1);
}

/****************************************************************************
  returns the header, footer and delimiters
  returns len if ok
          <0 otherwise
****************************************************************************/
static void read_header_and_footers (command, header, footer, delim)
char *command;
char **header;
char **footer;
char **delim;

{
output_command_struct *cur = output_command_head;

while (cur != NULL) {
  if (cur->command != NULL && strcasecmp (cur->command, command)==0) {
    *header = cur->header;
    *footer = cur->footer;
    *delim = cur->record_delim;
    return;
    }
  cur=cur->nxt;
  }
*header = NULL;
*footer = NULL;
*delim = NULL;
}

/***************************************************************************
  retrieve results from defined formats
***************************************************************************/
int display_results (records, cache, mode, auth_str, sub_query)
record_struct *records;
int cache;
char *mode;
char *auth_str;
int sub_query;

{
record_struct *ptr = records;
object_struct *object;
output_format_struct *output_format;
int multiple = FALSE, cnt=0;
char *output_type;
char *header;
char *footer;
char *rec_separator;

while (ptr != NULL && !multiple) {
#if DEBUG
  printf ("display_results: calling get_object_from_record with |%s|!\n",
     records->fields->data);
#endif
  object = get_object_from_record (ptr->fields);
#if DEBUG
  printf ("display_results: called get_object_from_record!!!\n");
#endif
  if (object != NULL && object->type != REFERRAL) {
    cnt++;
    if (cnt > 1)
      multiple = TRUE;
    }

  ptr = ptr->nxt;
#if DEBUG
  printf ("display_results: ptr is %p!\n", ptr);
#endif
  }
#if DEBUG
  printf ("display_results: doing second pass!!!\n");
#endif

/******
  Define the type of display
******/

if (mode)
  output_type = mode;
else if (multiple)
  output_type = default_multi;
else
  output_type = default_single;

read_header_and_footers (output_type, &header, &footer, &rec_separator);
if (header != NULL && cnt && !sub_query)
  printf ("%s\n", header);
cnt = 0;
while (records != NULL) {
#if DEBUG
  printf ("display_results: doing get_objct with |%s|\n", 
    records->fields->data);
#endif
  cnt++;
  object = get_object_from_record (records->fields);
#if DEBUG
if (object == NULL)
  printf ("problems here!!!\n");
printf ("display_results: have object |%s|\n", object->name);
#endif
  if (object != NULL) {
    if (rec_separator!=NULL && (!sub_query || cnt > 1)) 
      printf ("%s\n", rec_separator);
    if (strcasecmp (output_type, "dump") == 0)
      display_dump_format (object->name, records, cnt-1);
    else if ((output_format = retrieve_output_format (object->output_format,
      output_type)) == NULL) {
      print_error (MISC_DIAG, "No format for Object '%s'", object->name);
      display_dump_format (object->name, records, cnt-1);
      }
    else {
      display_record (output_format, object->name, records, rec_separator, 
        auth_str);
      }
    }
  else
    print_error (UNIDENT_ERROR, "can not find object!");
  if (cache)
    print_error (NON_AUTH_ANSWER,"");
   
  records=records->nxt;
  }
if (footer != NULL && cnt && !sub_query)
  printf ("%s\n", footer);
return (cnt);
}

/****************************************************************************
  returns a pointer to the specified format in command
            NULL if does not exist
****************************************************************************/
static output_format_struct *retrieve_output_format (display_list, command)
output_display_struct *display_list;
char *command;

{
int length;

length = strlen (command);
while (display_list != NULL) {
#if DEBUG
printf ("retrieve_output_format: looking at |%s|\n", display_list->command);
#endif
  if (length >= display_list->command_len &&
     strncasecmp (display_list->command, command, length)== 0)
    return (display_list->format);
  display_list = display_list->nxt;
  }
return (NULL);
}

/****************************************************************************
  prints out the record by decoding the output format
****************************************************************************/
static void display_record (output_format, object_name, record, delim, auth_str)
output_format_struct *output_format;
char *object_name;
record_struct *record;
char *delim;
char *auth_str;

{
llist_struct *deferred_queries, *top = NULL, *cur;

while (output_format != NULL) {
  switch (output_format->type) {
    case FORMAT_FIELD_DEF:
      if ((deferred_queries = display_field (output_format, object_name,
        record->fields, auth_str)) != NULL) {
        if (top == NULL)
          cur = top = deferred_queries;
        else {
          cur->nxt = deferred_queries;
          cur = deferred_queries;
          }
        }
      break;
    case FORMAT_TEXT:
      display_text (output_format->value, 0);
      break;
    case FORMAT_CONDIT_TEXT:
      display_conditional_text (output_format, record->fields);
      break;
    }
  output_format = output_format->nxt;
  }
cur = top;
while (cur != NULL) {
  if (delim!=NULL)
    printf ("%s\n", delim);
  run_sub_query (NULL, cur->datum);
  free (cur->datum);
  cur=cur->nxt;
  }
free_llist(top);
}

/****************************************************************************
  prints out the individual record
****************************************************************************/
static llist_struct *display_field (output_format, object_name, fields, 
  auth_str)
output_format_struct *output_format;
char *object_name;
field_struct *fields;
char *auth_str;

{
field_struct *cur= fields;
attribute_struct *attribute_info;
char *multi_delim;
char *fieldname;
int fixed_len, cnt, found = FALSE, sub_defer, sub_deref;
char *sub_format, *format_convert;
llist_struct *deferred_subqueries = NULL;
char datafldname[MAX_LINE];

/***
  first make sure exists
****/

cur = fields;
fieldname = output_format->value;
while (cur != NULL && !found) {
 strcpy (datafldname, get_attribute_name(cur->id));
 if (strcasecmp (datafldname, fieldname) == 0)
   found = TRUE;
 cur=cur->nxt;
 }
if (!found)
  return (NULL);

/****
  check out required fields are there! if not - return
****/

if (!required_fields_in_record (fields, output_format->condit_pos,
  output_format->condit_neg))
  return (NULL);

/*****
  go for it
*****/

multi_delim = output_format->multi_delim;
fixed_len = output_format->fixed_len;
sub_deref = output_format->sub_deref;
sub_defer = output_format->sub_defer;
sub_format = output_format->sub_format;
output_format = output_format->subitem;
format_convert = output_format->format_convert;

while (output_format != NULL) {
  switch (output_format->type) {
    case FORMAT_TEXT:
      display_text (output_format->value, 0);
      break;
    case FORMAT_AUTH_ELEM:
      display_text (auth_str, 0);
      break;
    case FORMAT_DATA:
      cur = fields;
      cnt = 0;
      while (cur != NULL) {

/*
        strcpy (datafldname, get_attribute_name(cur->id));
        if (strcasecmp (datafldname, fieldname) == 0 &&
          (attribute_info = get_attribute_info_from_name_and_object
          (object_name, datafldname))!=NULL) {
*/

        attribute_info = get_attribute_info_from_internal_id (cur->id);
#if DEBUG
printf ("display_field: have %d -> %s:%s -looking for |%s|\n", cur->id, 
  attribute_info->object_name, attribute_info->name, fieldname);
#endif
        if (strcasecmp (attribute_info->name, fieldname) == 0) { 
#if DEBUG
printf ("display_field: found! - print |%s|\n", cur->data);
#endif
          if (cnt > 0)
            display_text (multi_delim, 0);
          display_attribute (attribute_info, cur->data, auth_str, fixed_len, 
            format_convert, sub_deref, sub_defer, sub_format);
          if (sub_defer)
            deferred_subqueries = setup_deferred_subqueries 
              (deferred_subqueries, cur->data, sub_format);
          cnt++;
          }
        cur=cur->nxt;
        }
      break;
    default:
      printf ("display_field: oops don't have value %d\n", 
        output_format->type);
    }
  output_format = output_format->nxt;
  }
return (deferred_subqueries);
}

/****************************************************************************
  displays conditional text
****************************************************************************/
static void display_conditional_text (str, fields)
char *str;
field_struct *fields;

{
printf ("display_conditional_text: do this yet!\n");
}

/****************************************************************************
  this displays an attribute 
****************************************************************************/
static void display_attribute (attribute_info, data, auth_str, len, 
  format_convert, deref, defer, format)
attribute_struct *attribute_info;
char *data;
char *auth_str;
int len;
char *format_convert;
int deref;
int defer;
char *format;

{
switch (attribute_info->type) {
  case TYPE_TEXT:
    if (format_convert != NULL)
      display_text_w_format_convert (format_convert, data, len);
    else
      display_text (data, len);
    break;
  case TYPE_FILE:
    display_file (data);
    break;
  case TYPE_MIME:
    if (check_capabilities ("mime"))
      display_mime (data);
    break;
  case TYPE_SEE_ALSO:
    if (defer && !deref)
      return;
    do_see_also (data, format, defer, deref);
    break;
  case TYPE_REFERRAL:
    display_referral (data, auth_str);
    break;
  }
}

/****************************************************************************
  displays text string
****************************************************************************/
void display_text (str, fixed_len)
char *str;
int fixed_len;

{
char *c = str;
int cnt=0;

if (str == NULL)
  return;
while (*c != '\0') {
  c = display_char(c);
  c++; cnt++;
  }
if (fixed_len > 0 && cnt < fixed_len) 
  while (cnt < fixed_len) {
    display_char (" ");
    cnt++;
    }
}

/****************************************************************************
  displays a character
****************************************************************************/
static char *display_char(c)
char *c;

{
if (*c == '\\' && *(c+1) != '\0' && *(c+1) == 'n') { /* carrriage return */
  printf ("\n");
  c++;
  }
else
  printf ("%c", *c);
return (c);
}

/****************************************************************************
  opens file and displays contents
  returns TRUE on success
          FALSE on failure
****************************************************************************/
static int display_file (fname)
char *fname;
{
FILE *fp;
char line[MAX_LINE];

if ((fp = fopen (fname, "r")) == NULL) 
  return (FALSE);

while (fgets (line, MAX_LINE, fp) != NULL) {
  stripchar (line, '\n');
  printf ("%s\n", line);
  }
fclose (fp);
}

/****************************************************************************
  opens file and displays contents of mime file
  returns TRUE on success 
          FALSE on failure
****************************************************************************/
static int display_mime (fname)
char *fname;

{
printf ("%%display activate:mime\n");
display_file (fname);
printf ("%%display\n");
}

/****************************************************************************
  tries to find internal see also 
  if dereference on, prints only datum, does not try to do subquery
  expects in host:port:server_type:<query> 
          or host:port:<query> formats
****************************************************************************/
static void do_see_also (data, format, defer, deref)
char *data;
char *format;
int defer;
int deref;

{
char query[MAX_LINE];
char location[MAX_LINE];

separate_location_from_query (data, location, query);
if (deref || defer)
  display_text(query, 0);
else if (location_is_local (location)) 
  run_sub_query (format, query);
else  {
  if (format != NULL)
    printf ("%%see-also %s:%s %s\n", location, format, query);
  else
    printf ("%%see-also %s:%s\n", location, query);
  }
}

/****************************************************************************
 formats and starts the subquery 
****************************************************************************/
static int run_sub_query (format, query)
char *format;
char *query;
{
char sub_query[MAX_LINE];

if (format != NULL)
  sprintf (sub_query, "%s %s", format, query);
else
  strcpy (sub_query, query);
return (whois (sub_query, FALSE, TRUE));
}

/****************************************************************************
  prints out a referral record
  expects in host:port:server_type auth_str
****************************************************************************/
static void display_referral (data, auth_str)
char *data;
char *auth_str;

{
if (forward_status())
  forward_request (data, original_query(), auth_str);
else if (strncasecmp (data, "url", 3) == 0) /* url's do not have aa's */
  printf ("%%referral %s\n", data);
else if (auth_str != NULL)
  printf ("%%referral %s %s\n", data, auth_str);
else 
  printf ("%%referral %s\n", data);
}

/****************************************************************************
  gets the display status
****************************************************************************/
char *get_display_status(type)
char *type;

{
if (strcasecmp (type, "multi") == 0)
  return (default_multi);
return (default_single);
}

/****************************************************************************
  sets the display status assumes valid mode
  returns TRUE on success
          FALSE on failure
****************************************************************************/
int set_display_status (type, mode)
char *type, *mode;

{
char **ptr;

if (strcasecmp (type, "all") == 0) {
  set_display_status ("multi", mode);
  set_display_status ("single", mode);
  return (TRUE);
  }
else if (strcasecmp (type, "multi") == 0)
  ptr = &default_multi;
else if (strcasecmp (type, "single") == 0)
  ptr = &default_single;
else
  return (FALSE);
free (*ptr);
*ptr = NEW_STRING (mode);
return (TRUE);
}

/****************************************************************************
  sets up the the deferred subqueries
  returns a new element on the see-also list as a string value
****************************************************************************/
static llist_struct *setup_deferred_subqueries(list, data, format)
llist_struct *list;
char *data;
char *format;
{
char str[MAX_LINE], word[MAX_LINE], *new, *cp, *next;

if (format == NULL)
  strcpy (str, data);
else {
  /* skip over referral part */
  cp = data;
  next = get_word (cp, word);
  sprintf (str, "%s %s %s", word, format, next);
  }
new = NEW_STRING (str);
list = append_llist (list, new);
return (list);
}

/****************************************************************************
  If condit_pos set, makes sure that condit_pos field in record
  If condit_neg set, make sure that condit_neg field not in record
  returns TRUE on success
          FALSE on failure
****************************************************************************/
static int required_fields_in_record (fields, condit_pos, condit_neg)
field_struct *fields;
char *condit_pos;
char *condit_neg;

{
attribute_struct *attribute_info;
field_struct *cur;
char *fieldname;
int found;

if (condit_pos == NULL && condit_neg == NULL)
  return (TRUE);

if (condit_pos != NULL) {
  cur = fields;
  fieldname = condit_pos;
  found = FALSE;
  while (cur != NULL && !found) {
    if ((attribute_info = get_attribute_info_from_internal_id(cur->id))!=NULL)
      if (strcasecmp (fieldname, attribute_info->name) == 0)
        found = TRUE;
    cur=cur->nxt;
    }
  if (!found)
    return (FALSE);
  }
if (condit_neg != NULL) {
  cur = fields;
  fieldname = condit_neg;
  found = FALSE;
  while (cur != NULL && !found) {
    if ((attribute_info = get_attribute_info_from_internal_id(cur->id))!=NULL)
      if (strcasecmp (fieldname, attribute_info->name) == 0)
        found = TRUE;
    cur=cur->nxt;
    }
  if (found)
    return (FALSE);
  }
return (TRUE);
}

/****************************************************************************
  separates the location from the query
****************************************************************************/
separate_location_from_query (data, location, query)
char *data;
char *location;
char *query;

{
strcpy (query, get_word(data, location));
}

/****************************************************************************
  checks to see if fully qualified name and port equals localhost and 
  localport
****************************************************************************/
location_is_local (location)
char *location;
{
char datum[MAX_LINE], str[MAX_LINE];
char buf[MAX_LINE];
char local_host[MAX_LINE];
char local_port[MAX_LINE];

if (parse_line (location, local_host, datum)) {
  strcpy (buf, datum);
  if (!parse_line (buf, local_port, datum))
    strcpy (local_port, buf);
  if (strcasecmp (local_host, get_local_host_from_config()) == 0 &&
    strcmp (local_port, get_local_port_from_config()) == 0)
    return (TRUE);
  }
return (FALSE);
}
