/************************************************************************
RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and 
that no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
					scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
					markk@internic.net


Developed By: 
  Mark Kosters
***************************************************************************/

#include <stdio.h>
#include <time.h>
#include <varargs.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sys/socket.h>       /* socket stuff */
#include <netinet/in.h>
#include "defines.h"
#include "client_info.h"
#include "client_msgs.h"
#include "log.h"

/**************************************************************************
    log query information
**************************************************************************/
int log_entry (va_alist)
va_dcl
{
va_list ap;
FILE *fp;
char *format;
char *filename, *hostname;
int fd;

va_start(ap);
filename = va_arg (ap, char*);
format = va_arg(ap, char*);

if ((fp = fopen(filename, "a"))==NULL) {
  log_entry (RWHOIS_LOG_FILE, "ERROR save_into_log: could not open file %s", 
    filename);
  print_error (UNIDENT_ERROR,"");
  return (FALSE);
  }

fd = fileno (fp);

/** lock for exclusive use  - default is to wait **/
#if SUNOS /* interesting works if on SOLARIS if local file system */
if (flock (fd, LOCK_EX) != 0) {
printf ("prlems here |%s|\n", filename);
  log_entry (RWHOIS_LOG_FILE, "ERROR save_into_log: could not start lock %s", 
    filename);
  print_error (UNIDENT_ERROR,"");
  return (FALSE);
  }
#endif

timestamp(fp);
hostname = get_hostname(1);
fprintf(fp, " [%-15s] PID: %-8d", hostname, getpid());
vfprintf (fp, format, ap);
fprintf (fp, "\n");
#if SUNOS
if (flock (fd, LOCK_UN) != 0) {
  log_entry (RWHOIS_LOG_FILE, "ERROR save_into_log: could not release lock %s", 
    filename);
  return (FALSE);
  }
#endif
fclose(fp);
return (TRUE);
}

/**************************************************************************
    gets hostname
**************************************************************************/
char *get_hostname(g)
int g;                        /* peer socket */
{
static char buf[MAX_LINE];
static int tried_once = FALSE;
extern int errno;
struct sockaddr_in name;
int namelen = sizeof name;

if (tried_once)
  return (buf);
tried_once = TRUE;
if (getpeername(g, (struct sockaddr *)&name, &namelen) != 0)
  {
  print_error (FAILED_PEERNAME, "errno=%d", errno );
  sprintf(buf, "<FAILED>");
  }
else
  sprintf(buf, "%s", inet_ntoa(name.sin_addr) );
return (buf);
}

/**************************************************************************
    logs time of query
**************************************************************************/
void timestamp(f)
FILE *f;
{
time_t now;
struct tm *ts;
static char *months[] = {
    "Jan", "Feb", "Mar", "Apr", "May", "Jun",
    "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
};

now = time((time_t*)NULL);              /* current time */
ts = localtime(&now);           /* get now in pieces */
fprintf(f, "%2d-%s-%02d %02d:%02d:%02d", ts->tm_mday, months[ts->tm_mon],
  ts->tm_year % 100, ts->tm_hour, ts->tm_min, ts->tm_sec);
}
