/************************************************************************
RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and 
that no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By: 
  Mark Kosters
***************************************************************************/

#include <stdio.h>
#include <string.h>
#include "defines.h"
#include "client_msgs.h"
#include "main.h"
#include "read_config.h"
#include "deadman.h"
static char rcsid[] = "@(#)whois.c     1.2 1.2 4/21/93 10:03:29";

/************
  local globals 
************/
static char *default_dir;
static char *config_file;

#define USAGE_STR "usage: %s [-c config_file] [-r]\n"

/*************************************************************************
  main routine
**************************************************************************/
int main(argc, argv)
  int  argc;
  char *argv[];
{
  extern int    optind;
  extern char   *optarg;
  int           i = 0;
  int           root_server = FALSE;
  int           badopts = FALSE;
  int           do_chroot = TRUE;
  char c;

  default_dir = NULL;
  config_file = NULL;

  setbuf(stdout, NULL);
  while ((c = getopt(argc, argv, "c:rs")) != EOF) {
    switch (c) {
    case 'c':
      config_file = optarg;
      break;
    case 'r':
      root_server = TRUE;
      break;
    case 's':
      do_chroot = FALSE;
      break;
    default:
      badopts = TRUE;
      break;
    }
  }
  if (badopts) {
    usage(argv[0]);
  }

  /* default section */

  if (config_file == NULL)
    config_file = DEFAULT_RWHOIS_CONFIG_FILE;

  /* do 1-time initialization */
  if (!initialize_environment(config_file, FALSE))  
    exit (1);

  /* do security stuff -- mainly chroot()ing*/
  if (do_chroot) {
    if (!setup_security()) {
      fprintf(stderr, "Security setup failed! Exiting.\n");
      exit(1);
    }
  }
  else {
    fprintf(stderr, "Warning: chroot()ing turned off.\n");
  }

  if (is_daemon_server()) {
    run_daemon(root_server, get_local_host_from_config(), 
               atoi(get_local_port_from_config()));
  }
  else {
    run_session(root_server);
  }
  exit_program();
}

/*************************************************************************
  usage: prints the usage statement
**************************************************************************/
usage(prog_name)
  char *prog_name;
{
  fprintf(stderr, USAGE_STR, prog_name);
  exit(64);
}

/*************************************************************************
  runs the current session
**************************************************************************/
run_session(root_server)
  int root_server;

{
  char target[MAX_LINE];

  print_welcome_header();
  setlinebuf(stdin);
  setvbuf(stdout, (char *)NULL, _IONBF, 0);
  do {
    set_timer(get_deadman_time(), is_a_deadman);
    readline(target, MAX_LINE);
    stripchar(target, 13);
    unset_timer();
  }
  while (whois(target, root_server, FALSE));
}

/*************************************************************************
  prints the welcome header as you enter the system
**************************************************************************/
print_welcome_header()

{
  char hostname[MAX_HOSTNAME];

  printf("%%RWhois V-%s", RWHOIS_PROTOCOL_VERSION);
  if (strlen(strncpy(hostname, get_local_host_from_config(), MAX_HOSTNAME))!=0)
    printf(" %s", hostname);
  else
    printf(" UNKNOWN");
  printf(" (by InterNIC Registration Services V-%s)\n", RWHOIS_SERVER_VERSION);
  fflush(stdout);
}

/*************************************************************************
  exits the program
**************************************************************************/
exit_program()

{
  reset_signals();
  exit(0);
}

/*************************************************************************
  returns the config file
**************************************************************************/
char *get_config_file(is_chrooted)
  int is_chrooted;

{
  if (is_chrooted)
    return(strrchr(config_file, '/'));
  else
    return(config_file);
}
