/************************************************************************
RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and 
that no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
					scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
					markk@internic.net


Developed By: 
  Mark Kosters
***************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "defines.h"
#include "client_msgs.h"
#include "misc.h"

/**********************************************************************
  reads the line
***********************************************************************/
char *readline(buffer, size)
char *buffer;
int size;
{
char *cp;

if (!fgets(buffer, size, stdin))
  *buffer = '\0';
else
  {
  cp = buffer + strlen(buffer) - 1;
  if (*cp == '\n')
    *cp = '\0';
  }
return (buffer);
}

/************************************************************************
 Function: this precedure parses a <tag>: <datum> line
 returns TRUE if a record separator
         FALSE otherwise

 By: Mark Kosters
*************************************************************************/
int new_record (line)
char *line;

{
stripchar (line, '\n');
if (strcmp (line, "---") == 0)
  return (TRUE);
return (FALSE);
}

/************************************************************************
 Function: this precedure parses a <tag>: <datum> line

 By: Mark Kosters
*************************************************************************/
int parse_line (line, tag, datum)
char *line, *tag, *datum;

{
char *l, *t, *d;

stripchar (line, '\n');
if (strchr (line, ':'))
  {
  l = line;
  t = tag;
  while (*l !=  ':')
    *(t++) = *(l++);
  *t = '\0';
  l++; /* skip : */
  d = datum;
  while (*l != '\0' && *l != '\n')
    *(d++) = *(l++);
  *d = '\0';
  trim (tag);
  trim (datum);
  return (TRUE);
  }
*tag = '\0';
*datum = '\0';
return (FALSE);
}

/*************************************************************************
 Function: Strips out a specific character
**************************************************************************/
char *stripchar (str, ch)
char *str;
char ch;
{
int i, j;

for (i=0,j=0; i<strlen(str); i++)
  if (str[i] != ch)
    str[j++] = str[i];
  str[j] = '\0';
}

/************************************************************************
 Function: this procedure strips the blanks at both ends of the line

 By: Mark Kosters
*************************************************************************/
char *trim (str)
char *str;

{
Rtrim (str);
Ltrim (str);
}


/************************************************************************
 Function: this procedure strips the blanks at the end of the line

 By: Mark Kosters
*************************************************************************/
char *Rtrim ( str)
char *str;

{
int i;
for (i = strlen (str) - 1; str[i] == ' '; i--)
  str[i] = '\0';
}

/************************************************************************
 Function: this procedure strips the blanks at the beginning of the line

 By: Mark Kosters
 Fixed by Andy Linton (asjl@aarnet.edu.au)
*************************************************************************/
char *Ltrim( str)
char *str;

{
char *p1, *p2;

p1 = p2 = str;
while (*p1 == ' ' || *p1 == '\t') {
  p1++;
  }
if (p1 != p2) {
  while (*p2 != '\0') {
    *p2++ = *p1++;
    }
  }
}

/************************************************************************
 Function: this procedure puts the line on the form
*************************************************************************/
int wPutLine(fp, s)
  FILE *fp;
  char *s;
{
  int r = fputs(s, fp) != EOF && fputs(CRLF, fp) != EOF;
  fflush(fp);
  return r;
}

/************************************************************************
 Function: this procedure retrieves the cnt delim from mainbuf into cpbuf
  returns: 1 on success
           0 on failure

 By: Mark Kosters
*************************************************************************/
int delimstr(mainbuf, delim, cpbuf, cnt)
char *mainbuf, *delim, *cpbuf;
int cnt;
{
int i=0;
char *token;
static char *cpbuff = NULL;
static int cpbuff_size=0;

if (strlen (mainbuf)+1 > cpbuff_size) {
  if (cpbuff == NULL)
    cpbuff =  (char *)malloc ((strlen (mainbuf)+1)*sizeof(char));
  else
    cpbuff = (char *)realloc (cpbuff, strlen (mainbuf)+1);
  if (cpbuff == NULL) {
    print_error (MISC_DIAG, "delimstr: error requesting memory\n");
    return (FALSE);
    }
  cpbuff_size = strlen(mainbuf)+1;
  }
strcpy (cpbuff, mainbuf);
token = strtok (cpbuff, delim);
while (token != NULL && i < cnt) {
  i++;
  token = strtok (NULL, delim);
  }
if (token != NULL) {
  strcpy (cpbuf, token);
  return (TRUE);
  }
else
  return (FALSE);
}

/***************************************************************************
 returns 2 if alpha-numeric, "." "@",  "-", or "/"
****************************************************************************/
int mak_isalnum (ch)
char ch;

{
if (ch == '.' || ch == '@' || ch == '-' || ch == '/')
  return (2);
return (isalnum(ch));
}

/***************************************************************************
 reverses the string
****************************************************************************/
strrev(string)
char *string;
{
char *head, *tail, c;

for (head = string, tail = string + strlen(string) - 1; head < tail; 
  head++, tail--) {
  c = *head;
  *head = *tail;
  *tail = c;
  }
}

/***************************************************************************
 takes the first chars off the string
****************************************************************************/
Lstrtrunc (str, len)
char *str;
int len;

{
strrev (str);
str[len] = '\0';
strrev (str);
}

/****************************************************************************
 given a cp to a string, write a copy of the next "word" to the given
 buffer, skipping leading whitespace.  a "word" is some positive
 number of contiguous non-whitespace characters.  next_cp is set
 to point to after the word, and the function returns the word if found,
 else NULL (with the dest buffer unchanged).
****************************************************************************/
char *get_word(cp, buf)
char *cp, *buf;
{
char *original_buf = buf;

while (isspace(*cp)) cp++;
while (*cp && !isspace(*cp))
  *buf++ = *cp++;
if (buf != original_buf) {
  *buf = '\0';
  while (isspace(*cp)) cp++;
    return cp;
  }
else
  return NULL;
}


/****************************************************************************
 pastes the new info into the query string
****************************************************************************/
paste (line, cut_start, cut_end, rpl)
char *line;
char *cut_start, *cut_end;
char *rpl;

{
char  *cp = line;
char buf[MAX_LINE];
int i=0;

while ( cp <= cut_start) {
  buf[i++] = *cp;
  cp++;
  }
buf[i] = '\0';
strcat (buf, rpl);
strcat (buf, cut_end);
strcpy (line, buf);
}

/****************************************************************************
 does a case-insensitive string compare
****************************************************************************/
char *strSTR (str1, str2)       /*  by Jeff Odum  09/91  */
char *str1, *str2;
{
/*  Case insensitive sub-string search  */
int i,j, x;


if (!strlen(str1) || !strlen(str2))
  return ((char *) NULL);


for (x=0; str1[x]; x++)
  if (toupper(str1[x]) == toupper(str2[0]))
    {                            /*  Matched first letter */
    for (i=x, j=0; (str1[i] && str2[j]) &&
      (toupper(str1[i]) == toupper(str2[j])); i++, j++ ) ;
    if (!str2[j])               /*  Matched whole string  */
      return (&str1[x]);
    }
return ((char *) NULL);
}


#define MIN_SEPARATOR_LEN 10
#define SEPARATOR_CHAR '-'
/****************************************************************************
 returns if it is a separator line
 returns TRUE on success
         FALSE on failure
****************************************************************************/
int record_separator(line)
char *line;
{
if((strlen(line) > MIN_SEPARATOR_LEN)) {
  while (*line == SEPARATOR_CHAR)
    line++;
  if (*line == '\0') {
    return (TRUE);
    }
  }
return (FALSE);
}

char *strupr(a)
char *a;
{
char *b;
int i;

for (b=a ; *a != '\0'; a++ ) 
  *a = toupper(*a);
return (b);
}

/****************************************************************************
  returns char strings
****************************************************************************/
char *on_off(bool)
int bool;

{
static char *on = "ON", *off = "OFF";
if (bool)
  return (on);
else
  return (off);
}

/****************************************************************************
  returns TRUE if bool set to TRUE
          FALSE otherwise
****************************************************************************/
true_false(bool)
char *bool;

{
trim (bool);
if (strcasecmp (bool, "TRUE") == 0)
  return (TRUE);
else
  return (FALSE);
}

#include <errno.h>
#include <signal.h>

#define MAX_SET 20
/*************************************************************************
  runs the program
  returns TRUE on success
          FALSE on failure
  this is compliements of Scott
**************************************************************************/
int run_program (program, param)
char *program;
char *param;

{
int errno = 0;
extern char **environ;
char *myenviron[MAX_SET];
char *argv[MAX_SET];
char buf[MAX_LINE];
int i, pid;

signal( SIGINT, SIG_IGN);
signal( SIGQUIT, SIG_IGN);

/**
  enviroment stuff
**/

for (i=0; environ[i] != NULL && i < MAX_SET; i++) {
  myenviron[i] = NEW_STRING (environ[i]);
  strcpy (myenviron[i], environ[i]);
  }
myenviron[i] = NULL;
argv[0] = NEW_STRING (program);
i = 0;
while (delimstr (param, " \t", buf, i++) && i < MAX_SET) 
  argv[i] = NEW_STRING (buf);
argv[i] =  NULL;

pid = fork();

if ( pid == -1 ) {  /* Fork failed. */
  if ( errno == ENOMEM ) {
    print_error (MEMORY_ALLOCATION_PROBLEM, "");
#if DEBUG
    printf("fork: Not enough memory.\n");
    exit( -1 );
#endif
    }
  else if ( errno == EAGAIN ) {
    print_error (UNIDENT_ERROR, "");
#if DEBUG
    printf("fork: Not enough process slots.\n");
    exit( -1 );
#endif
    }
  else {
    print_error (UNIDENT_ERROR, "");
#if DEBUG
    printf("fork: errno %d.\n", errno);
    exit( -1 );
#endif
    }
  }
else if ( pid == 0 ) { /* Child. */
  execve ( argv[0], argv, myenviron);
  print_error (UNRECOV_ERROR, "");
#if DEBUG
  printf("Exec failed.( %s )\n", argv[0]);
#endif
  exit (-1);
  }
else {  /* Parent. */
  int wait_ret;
  while ( (wait_ret=wait(0)) != pid  && wait_ret != -1);
  /* setup the abort signals  */
  i = 0;
  while ( myenviron[i] != NULL)
    free (myenviron[i++]);
  i = 0;
  while ( argv[i] != NULL)
    free (argv[i++]);
  }
return (TRUE);
}

/*************************************************************************
* This function parses an <object_name>:<attribute_name>:<data> line
*
* Input: 'line'
* Sets: 'attribute_name', 'object_name' and 'data'
*
* returns TRUE if get tuple
*         FALSE if problems
**************************************************************************/
int get_tuple (object_name, attribute_name, data, line)
char *attribute_name;
char *object_name;
char *data;
char *line;
{
char str[MAX_LINE];

if (strlen (line) != 0 && parse_line (line, object_name, str)) {
  if (parse_line (str, attribute_name, data))
    return (TRUE);
  }
return (FALSE);
}
