/************************************************************************
RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and 
that no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By: 
  Mark Kosters
***************************************************************************/

#include <stdio.h>
#include <string.h>
#include <sys/stdtypes.h>
#include <sys/stat.h>
#ifdef POSIX
#include <stdlib.h>
#else
#include <malloc.h>
#endif
#include "defines.h"
#include "misc.h"
#include "client_msgs.h"
#include "records.h"
#include "commands.h"
#include "attributes.h"
#include "parse.h"
#include "objects.h"


/* Globals */

static object_struct *object_head = (object_struct *) NULL;
static llist_struct *object_by_type[NUM_OBJECTS];

/****
  local prototypes
****/
static void read_command_from_template();
static object_type get_object_type();
static void read_command_from_object();

/****************************************************************************
  reads the main configuration file only to get the object file and
  calls it (for rwaisindex). Note that is is a subset of read_config_file.
  returns TRUE if ok
          FALSE otherwise
****************************************************************************/
read_config_file_for_objects_only (config_file)
  char *config_file;

{
  FILE *fp;
  char line[BUFSIZ];
  char tag[MAX_TEMPLATE_DESC];
  char datum[MAX_TEMPLATE_DESC];
  int retval = FALSE;

  if((fp = fopen(config_file, "r")) == (FILE *) NULL){
    print_error (MISC_DIAG, "Config file %s not found", config_file);
    return(FALSE);
  }
  while ((fgets (line, BUFSIZ, fp)) != NULL) {
    if (parse_line (line, tag, datum)) {
      if (strcasecmp (tag, "object-file") == 0)
        retval = read_objects (datum);
    }
  }
  fclose (fp);
  return (retval);
}

/****************************************************************************
  reads the main configuration file for objects
  returns TRUE if ok
          FALSE otherwise
****************************************************************************/
int read_objects(config_file)
  char *config_file;

{
  FILE *fp;
  char line[BUFSIZ];
  char tag[MAX_TEMPLATE_DESC];
  char datum[MAX_TEMPLATE_DESC];
  int retval = TRUE;
  object_struct *cur, *new;
  struct stat buf;
  int fnum;

  if((fp = fopen(config_file, "r")) == (FILE *) NULL){
    print_error (MISC_DIAG, "Object file '%s' not found", config_file);
    return(FALSE);
  }

  while ((fgets (line, BUFSIZ, fp)) != NULL) {
    if (new_record (line) || object_head == NULL)
    {
      if (object_head != NULL) 
        if (!read_attributes (cur)) 
          retval = FALSE;
      /* make new template record */
      if((new = NEW_OBJECT) == (object_struct *)NULL) {
        print_error (MEMORY_ALLOCATION_PROBLEM, "");
        return(FALSE);
      }
      if (object_head == NULL)
        cur = object_head = new;
      else {
        cur->nxt = new;
        cur = new;
      }
    }

    if (parse_line (line, tag, datum)) {
      if (strcasecmp (tag, O_NAME) == 0)
        cur->name = NEW_STRING (datum);
      else if (strcasecmp (tag, O_ATTRIBUTEDEF) == 0)
        cur->attribute_fname = NEW_STRING (datum);
      else if (strcasecmp (tag, O_LOCALDB) == 0 ||
               strcasecmp (tag, "waisdb") ==0)
        cur->local_db = NEW_STRING (datum);
      else if (strcasecmp (tag, O_CACHEDB) == 0)
        cur->cache_db = NEW_STRING (datum);
      else if (strcasecmp (tag, O_DBDIR) == 0)
        cur->dbdir = NEW_STRING (datum);
      else if (strcasecmp (tag, O_DISPLAY) == 0)
        cur->output_format
          = read_display_format_from_object(cur->output_format,
                                            datum);
      else if (strcasecmp (tag, O_DESCRIPTION) == 0)
        cur->description = NEW_STRING (datum);
      else if (strcasecmp (tag, O_COMMAND) == 0)
        read_command_from_object (cur, datum);
      else if (strcasecmp (tag, O_TYPE) == 0 && strlen (datum))
        cur->type = get_object_type (datum);
    }
  }

  if (!read_attributes (cur))
    retval = FALSE;
  if (retval)
    link_objects_by_type();
  fclose (fp);
  return (retval);
}

/****************************************************************************
  sets the object type
****************************************************************************/
static object_type get_object_type (type)
  char *type;

{
  trim (type);

  if (strcasecmp (type, "OBJECT")==0)
    return (OBJECT);
  else if (strcasecmp (type, "REFERRAL") == 0)
    return (REFERRAL);
  /**
    this is being disposed of. 12/1/94

    else if (strcasecmp (type, "ALL_OBJECTS")==0)
    return (ALL_OBJECTS);
    **/
  else if (strcasecmp (type, "SOA")==0)
    return (SOA);
  print_error (MISC_DIAG,
               "Unknown template type %s in object definition file", 
               type);
  return (NUM_OBJECTS);
}

/****************************************************************************
  links the various templates by type
****************************************************************************/
link_objects_by_type ()

{
  int i;
  object_struct *cur;

  /** initialize to NULL **/
  for ( i=0; i< NUM_OBJECTS; i++)
    object_by_type[i] = NULL;

  /** run through the types **/
  cur = object_head;
  while (cur != NULL) {
    object_by_type[cur->type] = append_llist (object_by_type[cur->type],
                                              cur);
    if (cur->type == OBJECT) 
      object_by_type[ALL_OBJECTS] = append_llist (object_by_type[ALL_OBJECTS],
                                                  cur);
    object_by_type[EVERY_OBJECT] = append_llist (object_by_type[EVERY_OBJECT],
                                                 cur);
    cur=cur->nxt;
  }
}

/****************************************************************************
  returns a llist of objects specified by type
****************************************************************************/
llist_struct *return_objects_by_type (type)
  object_type type;

{
  return (object_by_type[type]);
}

/****************************************************************************
  returns all the objects defined
****************************************************************************/
llist_struct *return_all_defined_objects(str)
  char *str;

{
  object_struct *object;

  if (strlen (str) && (object = return_object_by_command (str)) != NULL)
    return (append_llist (NULL, object));
  else if (strlen(str) == 0)
    return (object_by_type[EVERY_OBJECT]);
  return (NULL);
}

/****************************************************************************
  returns attributes of object by name
****************************************************************************/
attribute_struct *get_object_attributes_by_object_name (object_name)
  char *object_name;
{
  object_struct *cur;

  cur = object_head;
  while (cur != NULL) {
    if (strcasecmp (cur->name, object_name) == 0)
      return (cur->attributes);
    cur = cur-> nxt;
  }
  return (NULL);
}

/****************************************************************************
  returns object by name
****************************************************************************/
object_struct *get_object_by_object_name (object_name)
  char *object_name;
{
  object_struct *cur;

  cur = object_head;
  while (cur != NULL) {
    if (strcasecmp (cur->name, object_name) == 0)
      return (cur);
    cur = cur-> nxt;
  }
  return (NULL);
}

/***************************************************************************n
  reads the command structure from the template
  places it on the command list
****************************************************************************/
static void read_command_from_object (object, str)
  object_struct *object;
  char *str;

{
  char command[MAX_TEMPLATE_DESC];
  char length_str[MAX_TEMPLATE_DESC];
  int length;
  old_command_struct *new, *cur;

  if (parse_line (str, command, length_str)) 
    length = atoi (length_str);
  else {
    strcpy (command, str);
    length = strlen(command);
  }
  append_command (command, length, NULL, NULL, TRUE, COMMAND_OBJECT, object,
                  NULL, NULL);
}

/****************************************************************************
  builds linked list or returns a pointer to linked list
  
  Returns:templates that match criterion
          NULL if none exist
****************************************************************************/
llist_struct *setup_search_objects (query_item, type)
  query_item_struct *query_item;
object_type type;

{
  query_item_struct *cur;

  if (type == REFERRAL || type == SOA)
    return (return_objects_by_type (type));

  /**
    first pass - check to see if any object pointers defined  - return it
    **/

  cur = query_item;
  while (cur != NULL) {
    if (cur->object != NULL)
      return (append_llist (NULL, cur->object));
    cur=cur->nxt;
  }

  /**
    second pass - check to see if any attributes are unique - return it
    fix me - I'm punting for know - will come back!
    MAK
    ***/

  cur=query_item;
  while (cur != NULL)
    return (return_objects_by_type (type));
}

/****************************************************************************
  returns the object based on the first field (should be 
   Object-type: [object])
  NULL if world is on the right place to be
****************************************************************************/
object_struct *get_object_from_record (fields)
  field_struct *fields;

{
  attribute_struct *attrib;

  if ((attrib = get_attribute_info_from_internal_id (fields->id)) != NULL) 
    return (return_object_by_command (attrib->object_name));
  return (NULL);
}

/****************************************************************************
  returns the pointer to the attributes of that designated object based
  on first tag
  returns ptr if exists
          NULL if not
****************************************************************************/
attribute_struct *get_object_attributes_by_tag (tag, datum)
  char *tag;
  char *datum;

{
  llist_struct *object_list;
  object_struct *object;
  attribute_struct *attrib;
  int found = FALSE;

  object_list = return_all_defined_objects (datum);
  while (object_list !=  NULL && !found) {
    object = object_list->datum;
    attrib = object->attributes;
    while (attrib != NULL && !found)
      if ((strcasecmp (tag, attrib->name) == 0 ||
           strcasecmp (tag, attrib->name_alias) == 0) && attrib->def_object)
        found = TRUE;
      else
        attrib=attrib->nxt;
    object_list = object_list->nxt;
  }
  if (found) 
    return (object->attributes);
  else
    return (NULL);
}

/****************************************************************************
  debugs the objects
****************************************************************************/
debug_print_objects()

{
  object_struct *cur = object_head;

  while (cur != NULL) {
    printf ("name: |%s|\n", cur ->name);
    printf ("attribute file: |%s|\n", cur -> attribute_fname);
    printf ("local wais database: |%s|\n", cur -> local_db);
    printf ("cache wais database: |%s|\n", cur -> cache_db);
    printf ("description: |%s|\n", cur -> description);
    printf ("type: %d\n", cur -> type);
    debug_print_attributes (cur->attributes);
    printf ("---\n");
    cur = cur->nxt;
  }
  printf ("debug_print_objects: done!\n");
}
