/************************************************************************
RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and 
that no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
					scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
					markk@internic.net


Developed By: 
  Mark Kosters
***************************************************************************/
#ifndef _OBJECTS_H_
#define _OBJECTS_H_

#include <sys/stdtypes.h>
#include "conf.h"
#include "attributes.h"
#include "output.h"
#include "displays.h"

#define	 MAX_FILENAME         256
#define	 MAX_TEMPLATE_NAME    256
#define	 MAX_TEMPLATE_DESC    256

/****
  object config file tags
****/

#define O_NAME         "name"
#define O_ATTRIBUTEDEF "attributedef"
#define O_LOCALDB      "localdb"
#define O_CACHEDB      "cachedb"
#define O_DBDIR        "dbdir"
#define O_DISPLAY      "display"
#define O_DESCRIPTION  "description"
#define O_COMMAND      "command"
#define O_TYPE         "type"

/****
  object types
*****/

typedef enum {
    OBJECT,                  /* an object */
    REFERRAL,                /* an referral object */
    ALL_OBJECTS,             /* all objects (non-referrals, non-soa) */
    SOA,                     /* an soa record */
    EVERY_OBJECT,            /* every object (referral/soa/data) */
    NUM_OBJECTS              /* total number of objects (don't delete) */
} object_type;

/*****
  object structure
*****/

typedef struct _object_struct{
  char	 *name;                             /* name of object */
  int	 num;                               /* number of object */
  char	 *attribute_fname;                  /* attribute template fname */
  char	 *local_db;                         /* local wais db fname */
  char	 *cache_db;                         /* cache wais db fname */
  char	 *dbdir;                            /* cached object directory */
  char	 *description;                      /* description of the database */
  int    type;                              /* type of object */
  attribute_struct *attributes;             /* ptr to list of attributes */
  output_display_struct *output_format;     /* ptr to list of output formats */
  void *commands;                           /* linked list of valid commands*/
  struct _object_struct *nxt;
  time_t	version;						/* version of schema */
} object_struct;

/***
  command list
***/

typedef struct _old_command_struct {
  char *command;
  int length;
  object_struct *object;
  struct _old_command_struct *nxt;
  } old_command_struct;

#define NEW_OBJECT (object_struct *) calloc ((unsigned)1, \
   (unsigned)sizeof(object_struct))
#define NEW_OLD_COMMAND (old_command_struct *) calloc ((unsigned)1, \
   (unsigned)sizeof(old_command_struct))

/***********
  prototypes
***********/
extern int read_object_config();
extern llist_struct *setup_search_objects();
extern llist_struct *return_all_defined_objects();
extern llist_struct *return_objects_by_type();
extern object_struct *get_object_from_record();
extern object_struct *get_object_by_object_name();
extern attribute_struct *get_object_attributes_by_tag ();
extern attribute_struct *get_object_attributes_by_object_name();

#endif
