/************************************************************************
RWhois Software

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and
that no charge is made for such copies.

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By:
  Mark Kosters
***************************************************************************/

#include <stdio.h>
#include <string.h>
#ifdef POSIX
#include <stdlib.h>
#else
#include <malloc.h>
#endif
#include "defines.h"
#include "client_msgs.h"
#include "output.h"

/******
  local prototypes
******/
static void save_output_format();
static output_format_type get_format_type();
static output_format_struct *make_subitem();

/**************************************************************************
  parses the output
***************************************************************************/
output_format_struct *encode_output_format (format_list, file)
output_format_struct *format_list;
char *file;

{
FILE *fp;
char line[BUFSIZ];
char tag[BUFSIZ];
char datum[BUFSIZ];
output_format_struct *cur, *new;

if ((fp = fopen (file, "r")) == NULL) {
  print_error (MISC_DIAG, "encode_output_format: open error on '%s'", file);
  return (NULL);
  }
while (fgets (line, BUFSIZ, fp) != NULL) {
  if (new_record (line) || format_list == NULL) {
    if((new = NEW_OUTPUT_FORMAT) == (output_format_struct *)NULL) {
      print_error (MEMORY_ALLOCATION_PROBLEM, "");
      return(NULL);
      }
    if (format_list == NULL)
      cur = format_list = new;
    else {
      cur->nxt = new;
      cur = new;
      }
    }
  if (parse_line (line, tag, datum)) {
    stripchar (datum, '"');
    save_output_format (cur, tag, datum);
    }
  }
fclose (fp);
return (format_list);
}

/**************************************************************************
  saves the output to a structure
***************************************************************************/
static void save_output_format (format_list, tag, datum)
output_format_struct *format_list;
char *tag;
char *datum;

{
if (strcasecmp (tag, O_TYPE) == 0)
  format_list->type = get_format_type(datum);
else if (strcasecmp (tag, O_FIELD) == 0)
  format_list->value = NEW_STRING(datum);
else if (strcasecmp (tag, O_FIXED_LEN) == 0)
  format_list->fixed_len = atoi(datum);
else if (strcasecmp (tag, O_SUB_DEREF) == 0)
  format_list->sub_deref = true_false(datum);
else if (strcasecmp (tag, O_SUB_DEFER) == 0)
  format_list->sub_defer = true_false(datum);
else if (strcasecmp (tag, O_CONDIT_POS) == 0)
  format_list->condit_pos = NEW_STRING (datum);
else if (strcasecmp (tag, O_CONDIT_NEG) == 0)
  format_list->condit_neg = NEW_STRING (datum);
else if (strcasecmp (tag, O_FRMT_CONVRT) == 0)
  format_list->format_convert = NEW_STRING (datum);
else if (strcasecmp (tag, O_SUB_FORMAT) == 0)
  format_list->sub_format = NEW_STRING (datum);
else if (strcasecmp (tag, O_TITLE) == 0)
  format_list->subitem = make_subitem (format_list->subitem, tag, datum);
else if (strcasecmp (tag, O_DATA) == 0)
  format_list->subitem = make_subitem (format_list->subitem, tag, datum);
else if (strcasecmp (tag, O_AUTH_ELEM) == 0)
  format_list->subitem = make_subitem (format_list->subitem, tag, datum);
else if (strcasecmp (tag, O_MULTI_DELIM) == 0)
  format_list->multi_delim = NEW_STRING (datum);
}

/**************************************************************************
  determines the field type
***************************************************************************/
static output_format_type get_format_type (type)
char *type;

{
if (strcasecmp (type, O_FIELD_DEF)==0)
  return (FORMAT_FIELD_DEF);
else if (strcasecmp (type, O_TEXT_DEF) == 0)
  return (FORMAT_TEXT);
else if (strcasecmp (type, O_CONDIT_TEXT)==0)
  return (FORMAT_CONDIT_TEXT);
else if (strcasecmp (type, O_AUTH_ELEM)==0)
  return (FORMAT_AUTH_ELEM);
print_error (UNIDENT_ERROR, "get_format_type: type %s not found!", type);
}

/**************************************************************************
  appends subitem to list
***************************************************************************/
static output_format_struct *make_subitem (ptr, type_des, value)
output_format_struct *ptr;
char *type_des;
char *value;

{
output_format_struct *new, *cur = ptr;

new = NEW_OUTPUT_FORMAT;
if (strcasecmp (type_des, O_TITLE)==0)
  new->type = FORMAT_TEXT;
else if (strcasecmp (type_des, O_DATA)==0)
  new->type = FORMAT_DATA;
else if (strcasecmp (type_des, O_SUB_FORMAT)==0)
  new->type = FORMAT_SUB_FORMAT;
else if (strcasecmp (type_des, O_AUTH_ELEM)==0)
  new->type = FORMAT_AUTH_ELEM;
new->value = NEW_STRING (value);
if (cur == NULL)
 return (new);
while (cur->nxt != NULL)
  cur=cur->nxt;
cur->nxt = new;
return (ptr);
}
/**************************************************************************
  debugs the format list
***************************************************************************/
debug_output_format (cur)
output_format_struct *cur;

{
while (cur != NULL) {
  printf ("type: %d\n", cur->type);
  printf ("value: %s\n", cur->value);
  printf ("multi_delim: %s\n", cur->multi_delim);
  debug_subitems (cur->subitem);
  cur=cur->nxt;
  printf ("----\n");
  }
}

/****************************************************************************
  debugs subitem format list
****************************************************************************/
debug_subitems (cur)
output_format_struct *cur;

{
while (cur!= NULL) {
  printf ("sub type: %d\n", cur->type);
  printf ("sub value: %s\n", cur->value);
  printf ("sub multi_delim: %s\n", cur->multi_delim);
  cur=cur->nxt;
  }
}
