/************************************************************************
RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and 
that no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
					scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
					markk@internic.net


Developed By: 
  Mark Kosters
***************************************************************************/

#include "conf.h"

#ifndef _OUTPUT_H_
#define _OUTPUT_H_

/****
  output format tokens
****/

#define O_TYPE         "type"
#define O_FIELD        "field"
#define O_FIXED_LEN    "fixed-len"
#define O_TITLE        "title"
#define O_DATA         "data"
#define O_MULTI_DELIM  "multi-delim"
#define O_SUB_FORMAT   "sub-format"
#define O_SUB_DEFER    "sub-defer"
#define O_SUB_DEREF    "sub-deref"
#define O_CONDIT_POS   "condit-pos"
#define O_CONDIT_NEG   "condit-neg"
#define O_FRMT_CONVRT  "format-convert"

#define O_FIELD_DEF     "field"
#define O_TEXT_DEF      "text"
#define O_CONDIT_TEXT   "condit-text"
#define O_AUTH_ELEM     "auth-elem"

typedef enum {
  FORMAT_FIELD_DEF,                   /* field definition */
  FORMAT_TEXT,                        /* text definition */
  FORMAT_CONDIT_TEXT,                 /* conditional text */
  FORMAT_AUTH_ELEM,                   /* print out the authority elem (soa) */
  FORMAT_DATA,                        /* data definition (placement) */
  FORMAT_SUB_FORMAT,                  /* definition of display for see-also */
  FORMAT_SUB_DEFER,                   /* defer for see-also */
  FORMAT_SUB_DEREF                    /* defererence a see-also */
  } output_format_type;

typedef struct _output_format_struct {
  output_format_type type;               /* type of format */
  char *value;                           /* value of element (record) */
  char *multi_delim;                     /* delimiter for multi-valued fields */
  int  fixed_len;                        /* fixed length for element */
  int  sub_deref;                        /* dereferenece a see-also */
  int  sub_defer;                        /* defer referencing a see-also */
  char *sub_format;                      /* format of see-also */
  char *condit_pos;                      /* requires attribute */
  char *condit_neg;                      /* only if attribute not there */
  char *format_convert;                  /* format conversion utility */
  struct _output_format_struct *subitem; /* subitems for positioning */
  struct _output_format_struct *nxt;     /* next element on list */
  } output_format_struct;

#define NEW_OUTPUT_FORMAT (output_format_struct *) calloc ((unsigned)1, \
   (unsigned)sizeof(output_format_struct))

/*******
  prototypes
*******/

extern output_format_struct *encode_output_format();
extern char *decode_output_format();
#endif
