/************************************************************************
RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and 
that no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
					scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
					markk@internic.net


Developed By: 
  Mark Kosters
***************************************************************************/

#include <stdio.h>
#include <string.h>
#include <strings.h>
#ifdef POSIX
#include <stdlib.h>
#else
#include <malloc.h>
#endif
#include "defines.h"
#include "records.h"


#define MAX_WAIS_RECORD 25600

/*****
  local prototypes
******/

field_struct *read_fields_from_text();
field_struct *add_field();

/****************************************************************************
  reads the text coming back from the database engine
  returns ptr to top record
****************************************************************************/
record_struct *read_record_from_text (top_record, txt, sort_order)
record_struct *top_record;
char *txt;
int (*sort_order)();

{
record_struct *new, *cur = NULL;

new = NEW_RECORD;

new->fields = read_fields_from_text (txt);
if (top_record == NULL)
  top_record = new;
else {
  cur = top_record;
  while (cur->nxt != NULL)
    cur = cur->nxt;
  cur->nxt = new;
  }
return (top_record);
}

/****************************************************************************
  turns txt into attribute/value combos
  returns ptr to first attribute
****************************************************************************/
field_struct *read_fields_from_text (txt)
char *txt;

{
field_struct *fields = NULL;
char attribute_name[MAX_LINE];
char object_name[MAX_LINE];
char datum[MAX_LINE];
char line[MAX_LINE];
int retval;
char *cptr;
int cnt=0;

if (debug_status())
  printf ("read_fields_from_text: have |%s|\n", txt);

while (delimstr (txt, "\n", line, cnt++)) {
  for(cptr = line; *cptr != '\0'; cptr++)   /* Convert tabs to spaces */
    if((*cptr != ' ') && isspace(*cptr))
      *cptr = ' ';
  if (get_tuple (object_name, attribute_name, datum, line))
    fields = add_field (fields, object_name, attribute_name, datum);
  }
return(fields);
}

/****************************************************************************
  turns txt into attribute/value combos
  returns ptr to first attribute
****************************************************************************/
field_struct *add_field (fields, object_name, attribute_name, datum)
field_struct *fields;
char *object_name;
char *attribute_name;
char *datum;

{
field_struct *cur, *new;
int i;

new = NEW_FIELD;
if ((new->id = get_id_from_object_and_name(object_name, attribute_name))==-1) {
  free (new);
  if (debug_status())
    printf ("add_field: attribute |%s| under |%s| not defined!\n", 
      attribute_name, object_name);
  return (fields);
  }
new->data = NEW_STRING (datum);
#if DEBUG
printf ("add_field: have id %d for |%s| -> |%s|\n", new->id, 
  attribute_name, new->data);
#endif

if (fields == NULL)
  return (new);
cur = fields;
while (cur->nxt != NULL)
  cur=cur->nxt;
cur->nxt = new;
return (fields);
}

/****************************************************************************
  releases memory
****************************************************************************/
free_records(records)
record_struct *records;

{
record_struct *delrec;

while (records != NULL) {
  free_fields (records->fields);
  delrec = records;
  free (delrec);
  records = records->nxt;
  }
}

/****************************************************************************
  releases memory
****************************************************************************/
free_fields(fields)
field_struct *fields;

{
field_struct *delfield;

while (fields != NULL) {
  free (fields->data);
  delfield = fields;
  fields = fields->nxt;
  free (delfield);
  }
}


/****************************************************************************
  debugging stuff
****************************************************************************/
debug_records(records)
record_struct *records;

{
field_struct *fields;

while (records != NULL) {
  fields = records->fields;
  while (fields!= NULL) {
    printf ("debug_records: field_id %d\n", fields->id);
    fields = fields->nxt;
    }
  printf ("debug_records: ---\n");
  records = records->nxt;
  }
}
