/************************************************************************
RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and 
that no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
					scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
					markk@internic.net


Developed By: 
  Mark Kosters
***************************************************************************/

#include <stdio.h>
#include <string.h>
#include "defines.h"
#include "misc.h"
#include "parse.h"
#include "whois.h"

#define CHECK 1
#define REDUCE_IT 2

/**
 local prototypes
***/

void bump_off_userid();
void bump_off_domain();
void bump_off_net_prefix();
static unsigned long addtolong();
static char *longtoaddstr();
static unsigned long mask_addr_to_len();
static int parse_for_netlen();
static int strip_netlen();

/******************************************************************************
  is the query reducible
  returns TRUE if can make reduction 
          FALSE if not
******************************************************************************/
int is_query_reducible (query_list, pass)
query_item_struct *query_list;
int pass;

{
int retval = -1;
char *c;

while (query_list != NULL)
  {
  c = query_list->value;
  while (*c != '\0')
    if (!mak_isalnum(*c++))
      *query_list->value = '\0';
  if (is_email_address (query_list, pass, CHECK))
    retval = KEEPON_ACTION;
  else if (is_ip_network(query_list, pass, CHECK)) {
    retval = KEEPON_ACTION;
    }
  else if (is_domain(query_list, pass, CHECK)) {
    retval = KEEPON_ACTION;
    }
  query_list=query_list->nxt;
  }
if (pass == 1 && retval == -1)
  return (WHOIS_PP_ACTION);
else if (retval == -1)
  return (PUNT_ACTION);
return (retval);
}

/******************************************************************************
  this procedure modifies the query to get a referral
  does not make a reduction on pass 1
  returns TRUE if reduction made
          FALSE if not
******************************************************************************/
reduce_query (query_list, pass)
query_item_struct *query_list;
int pass;

{
int reduction_made = FALSE;

while (query_list != NULL)
  {
  if (is_email_address (query_list, pass, REDUCE_IT) && pass > 1) {
    bump_off_userid(query_list->value);
    query_list->attribute = NEW_STRING ("Domain-Name");
    reduction_made = TRUE;
    }
  else if (is_ip_network(query_list, pass, REDUCE_IT)) {
    if (pass > 1)
      bump_off_net_prefix (query_list->value);
    query_list->attribute = NEW_STRING ("IP-Network");
    reduction_made = TRUE;
    }
  else if (is_domain(query_list, pass, REDUCE_IT)) {
    if (pass > 1)
      bump_off_domain (query_list->value);
    query_list->attribute = NEW_STRING ("Domain-Name");
    reduction_made = TRUE;
    }
  query_list=query_list->nxt;
  }
return (reduction_made);
}

/******************************************************************************
  this function checks to see if it is a email address
  returns TRUE if can reduce query
          FALSE if can not
******************************************************************************/
int is_email_address (qitem, pass, mode)
query_item_struct *qitem;
int pass;
int mode;

{
if (strchr (qitem->value, '@'))
  return (TRUE);
return (FALSE);
}

/******************************************************************************
  this procedure strips off the userid on a rfc822 email address
******************************************************************************/
void bump_off_userid (str)
char *str;

{
char buf[MAX_LINE];
strcpy (buf, strchr(str,'@')+1);
strcpy (str, buf);
}

/******************************************************************************
  this function checks to see if it is a domain name
  returns TRUE if is a domain name
          FALSE if not
******************************************************************************/
int is_domain (qitem, pass, mode)
query_item_struct *qitem;
int pass;
int mode;

{
char *c = qitem->value;

if (strchr(qitem->value, '.')==NULL) {
  if (strlen (qitem->value) != 2 || (qitem->loop_detect && mode == CHECK) ) {
    return (FALSE);
    }
  else if (strlen(qitem->value) == 2) { 
#ifdef USE_WAIS
    strcat (qitem->value, ".");
    qitem->loop_detect = TRUE;
#else
    return (FALSE);
#endif
    }
  }
else if (strlen (strchr (qitem->value, '.')) == 1 && 
  strlen (qitem->value) > 3)
  return (FALSE);
else if (qitem->loop_detect && mode == CHECK)
  return (FALSE);

while ( *c != '\0' && (isalnum(*c) || *c == '.' || *c == '-'))
  c++;
if (*c == '\0')
  return (TRUE);
return (FALSE);
}

/******************************************************************************
  this procedure bumps off the last octet if not found
******************************************************************************/
void bump_off_domain (str)
char *str;

{
/** 
  for top-level stuff 
***/
if (strlen (str) == 3 && str[strlen(str)-1] == '.') {
#ifndef USE_WAIS
  str[strlen(str)-1] = '\0';
#endif
  return;
  }
strrev (str);
*strrchr(str,'.') = '\0';
strrev (str);
}

/******************************************************************************
  this function checks to see if it is a ip number
  returns TRUE if is a ip number
          FALSE if not
******************************************************************************/
int is_ip_network (qitem, pass, mode)
query_item_struct *qitem;
int pass;
int mode;

{
char *c = qitem->value;
int prefix;

while ( *c != '\0' && (isdigit(*c) || *c == '.' || *c == '/'))
  c++;
if (*c != '\0')
  return (FALSE);

prefix = parse_for_netlen(qitem->value);
if (prefix  <= 8) {
  if (qitem->loop_detect == TRUE && mode == CHECK) {
    return (FALSE);
    }
  else 
    qitem->loop_detect = TRUE;
  }
return (TRUE);
}

/******************************************************************************
  this procedure bumps off the last prefix if not found
  if prefix already three
******************************************************************************/
void bump_off_net_prefix (str)
char *str;

{
long addr;
char buf[18];
int netlen = parse_for_netlen (str);

if (!strip_netlen (str))
  netlen = 24;
else if (netlen > 24)
  netlen = 24;
else if (netlen > 8)
  netlen--;

addr = mask_addr_to_len (addtolong(str), netlen);
sprintf (str, "%s/%d", longtoaddstr(addr, buf), netlen);
}

/***********************************************************************
  parses network address to length
************************************************************************/
static int parse_for_netlen(addrstr)
char *addrstr;

{
int o1=0, o2=0, o3=0, o4=0;
int octsret;
int netlen;

if (rindex(addrstr, '/') != NULL)
  return (atoi (rindex(addrstr, '/')+1));
octsret = sscanf (addrstr, "%d.%d.%d.%d", &o1, &o2, &o3, &o4); 
if (octsret == 4 || octsret == 3)
  return (24);
else if (octsret == 2)
  return (16);
else 
  return (8);
}

/***********************************************************************
  strips the netlen
************************************************************************/
static int strip_netlen (str)
char *str;

{
if (rindex (str, '/') != NULL) {
  *strrchr (str, '/') = '\0';
  return (TRUE);
  }
return (FALSE);
}


/***********************************************************************
  Address Convert from str to long
************************************************************************/
static unsigned long addtolong (addstr)
char *addstr;

{
unsigned long addr;
int o1=0, o2=0, o3=0, o4=0;

sscanf (addstr, "%d.%d.%d.%d", &o1, &o2, &o3, &o4);
addr = (((unsigned)o1&0377)<<24) + (((unsigned)o2&0377)<<16) +
  (((unsigned)o3&0377)<<8) + ((unsigned) o4&0377);
return (addr);
}

/***********************************************************************
  Address Convert from long to str
************************************************************************/
static char *longtoaddstr(t, buf)
unsigned long t;
char *buf;

{
sprintf(buf, "%d.%d.%d.%d", (t)>>24, ((t) >>16) &0377, 
  ((t) >>8) &0377, t&0377);
return (buf);
}

/***********************************************************************
  Mask addr to prefix
************************************************************************/
static unsigned long mask_addr_to_len (addr, len)
unsigned long addr;
int len;

{
unsigned long mask = 0xffffffff;

mask = mask << (32-len);
return (addr&mask);
}
