/************************************************************************
RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and 
that no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By: 
  Mark Kosters
***************************************************************************/

#include <stdio.h>
#include "defines.h"
#include "misc.h"
#include "objects.h"
#include "schema.h"
#include "client_msgs.h"

/****************************************************************************
  displays the extended schema of the objects
  for diag purposes only
****************************************************************************/
display_schema (str)

{
  llist_struct *cur;
  llist_struct *tmp;
  object_struct *object;
  object_struct *tobj;
  int one_done = FALSE;
  int ver = FALSE;
  char tbuf[15];

  /*
   * Is this a request for version only?
   */

  cur = return_all_defined_objects(str);

  ver = is_this_version_only( str);

  while (cur != NULL) {
    object = cur->datum;
    if (object->attributes != NULL) {
      if (one_done == TRUE)
        printf ("%%schema\n");
      one_done=TRUE;
      if( ver == FALSE)
        display_attributes (object->name, object->attributes);
      else{
        if(!strftime( tbuf, 15, "%Y%m%d%H%M%S", localtime( &object->version)))
          printf("%%schema %s:version:0\n");
        else
          printf("%%schema %s:version:%s\n", object->name, tbuf);
      }
    }
    cur = cur->nxt;
  }
  if (!one_done)
    print_error (NO_OBJECT_SCHEMA_FOUND, "");
  else
    print_ok();
  set_passthru();
}

/****************************************************************************
  displays the attributes of a particular object
****************************************************************************/
display_attributes (name, cur)
  char *name;
attribute_struct *cur;

{
  int i = 0;
  while (cur != NULL) {
    if (i++!=0)
      printf ("%%schema\n");
    printf ("%%schema %s:attribute:%s\n", name, cur->name);
    if (cur->format != NULL)
      printf ("%%schema %s:format:%s\n", name, cur->format);
    printf ("%%schema %s:description:%s\n", name, cur->description);
    printf ("%%schema %s:indexed:%s\n", name, on_off (cur->index_words));
    printf ("%%schema %s:required:%s\n", name, on_off (cur->required));
    printf ("%%schema %s:mult-line:%s\n", name, on_off (cur->mult_lines));
    printf ("%%schema %s:type:%s\n", name, show_attribute_type (cur->type));
    printf ("%%schema %s:primary:%s\n", name, on_off (cur->primary_key));
    cur = cur->nxt;
  }
}

/****************************************************************************
  displays the extended schema of the objects
  for diag purposes only
****************************************************************************/
display_extended_schema (str)
  char *str;

{
  llist_struct *top, *cur;
  object_struct *object;
  int one_done = FALSE;

  top = cur = return_all_defined_objects(str);

  while (cur != NULL) {
    object = cur->datum;
    if (one_done)
      printf ("> ---\n");
    else
      printf ("FILE: tmpl.config\n");
    one_done = TRUE;
    printf ("> type: %d\n", object->type);
    printf ("> name: %s\n", object->name);
    printf ("> localdb: %s\n", object->local_db);
    printf ("> cachedb: %s\n", object->cache_db);
    if (object->attribute_fname != NULL)
      printf ("> attributedef: %s\n", object->attribute_fname);
    if (object->description != NULL)
      printf ("> description: %s\n", object->description);
    display_commands (object->commands);
    cur = cur->nxt;
  }

  cur = top;
  while (cur != NULL) {
    object = cur->datum;
    printf ("FILE: %s\n", object->attribute_fname);
    display_extended_attributes (object->attributes);
    cur = cur->nxt;
  }
  if (!one_done)
    print_error (NO_OBJECT_SCHEMA_FOUND, "");
  else
    print_ok();
  set_passthru();
}

/****************************************************************************
  displays the commands of a particular object
****************************************************************************/
display_commands (command)
  old_command_struct *command;

{
  while (command != NULL) {
    printf ("> command: %s:%d\n", command->command, command->length);
    command = command->nxt;
  }
}

/****************************************************************************
  displays the attributes of a particular object
****************************************************************************/
display_extended_attributes (cur)
  attribute_struct *cur;

{
  int i = 0;
  char buf[MAX_LINE];

  while (cur != NULL) {
    printf ("> %02d", i++);
    strcpy (buf, cur->name);
    if (cur->primary_key)
      strcat (buf, "*");
    printf (" %-25s", buf);
    printf (" %s\n", cur -> description);
    cur = cur->nxt;
  }
}
/****************************************************************************
    Check to see if this is a version request
***************************************************************************/
is_this_version_only( str)
  char *str;
{
  char *buf;
  char *next;

  buf = (char *)calloc( 1, strlen( str)+1);

  if(!( next = get_word( str, buf)))
    return( FALSE);

  if(!strncasecmp(next, "VERSION"))
    return(TRUE);

  return( FALSE);

}

