/************************************************************************
RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and 
that no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
					scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
					markk@internic.net


Developed By: 
  Mark Kosters
***************************************************************************/

#include <pwd.h>
#include "client_msgs.h"
#include "read_config.h"

#if AUTHORIZATION_ON
#include <stdio.h>
#include "defines.h"
#include "../tools/tcpd_wrapper/log_tcp.h"
#define PARANOID 1

/***************************************************************************
sets up a security playground by doing a chroot and setuid (taken
from W.Z. Venema's chrootuid code.

returns TRUE if ok
        FALSE otherwise
****************************************************************************/
setup_security ()
{
struct passwd *pwd;

/* make sure at the root rwhois area and Do the chroot() before giving 
   away root privileges. */
if (getuid() != 0) {
  print_error (MISC_DIAG, "need to run as root!");
  return (FALSE);
  }
/* change the user id of the process */
if ((pwd = getpwnam(get_process_userid())) == 0) {
  print_error (MISC_DIAG, "setup_security: %s user unknown", 
    get_process_userid());
  return (FALSE);
  }
if (chroot(get_root_dir())) {
  print_error (MISC_DIAG, "setup_security: chroot to (%s) returned false", 
    get_root_dir());
  return (FALSE);
  }

/* Switch group id then user id. */

if (setgid(pwd->pw_gid)) {
  print_error(MISC_DIAG, "setup_security: setgid failed");
  return (FALSE);
  }
if (setuid(pwd->pw_uid)) {
  print_error(MISC_DIAG, "setup_security: setuid failed");
  return (FALSE);
  }
/* In case we still have the /etc/passwd file still open. */
endpwent();
return (TRUE);
}

/****************************************************************************
 restricts who can run the command
   returns TRUE if can
           FALSE if not
****************************************************************************/
authorized_directive (directive)
char *directive;

{
struct from_host from;
int from_stat;
char *hosts_allow, *hosts_deny;

get_security_files (&hosts_allow, &hosts_deny);
#if PARANOID
from_stat = fromhost(&from);
if (from_stat == -1)
  return (FALSE);
#endif
return (hosts_access (directive, hosts_allow, hosts_deny, &from));
}
#endif
