/************************************************************************
RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and 
that no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
					scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
					markk@internic.net


Developed By: 
  Mark Kosters
***************************************************************************/


/* Copyright (c) CNIDR (Work in progress) */

/* WIDE AREA INFORMATION SERVER SOFTWARE:
   No guarantees or restrictions.  See the readme file for the full standard
   disclaimer.

   Brewster@think.com
*/

/* This is a simple shell user interface for generating wprot packets.
 * -brewster 7/90
 *
 * $Log: wais_interface.c,v $
 * Revision 1.1.1.1  1996/06/10 14:55:07  davidb
 * Initial baseline
 *
 * Revision 1.1  1993/02/16  15:09:27  freewais
 * Initial revision
 *
 * Revision 1.34  92/06/03  17:29:49  jonathan
 * Fixed so it exits on NULL input.
 * 
 * Revision 1.33  92/04/29  13:21:12  jonathan
 * Updated for release.
 * 
 * Revision 1.32  92/04/09  14:19:53  morris
 * now, specifying a port on the command line does a remote search on the
 * local machine at the specified port.  the previous behavior was to do a
 * local search, even when you specify a port.
 * 
 * Revision 1.31  92/03/17  14:39:28  jonathan
 * modified to work with new ui scheme.
 * 
 * Revision 1.30  92/03/06  14:50:34  jonathan
 * Correctly handle init_connection <= 0.
 * 
 * Revision 1.29  92/03/06  12:34:57  jonathan
 * Use result of init_connection to set message length.
 * 
 * Revision 1.28  92/02/29  20:04:32  jonathan
 * Fixed spelling of opening.
 * 
 * Revision 1.27  92/02/21  11:47:48  jonathan
 * fixed code to re-init properly (instead of using connect_to_server).
 * 
 * Revision 1.26  92/02/20  16:13:15  jonathan
 * Added -m for max results.
 * 
 * Revision 1.25  92/02/16  12:13:36  jonathan
 * Added domainname to init message.
 * 
 *
 * Important functions:
 *   display_search_response
 *   main
 *
 */

/* to do:
 *  fix the number parser in view document number
 *  fix if <cr> put to view document number, same as q
 *  show the index again after viewing a document
 *  do a 'more' type thing on viewing the doc  
 *  do something to allow relevance feedback
 */

/* Unix includes */

#include <ctype.h>
#include <string.h>
#include <unistd.h>
#include <strings.h>

/* WAIS includes */

#include <ui.h>
#include <sockets.h>
#include <docid.h>

/* local includes */

#include "records.h"
#include "wais_interface.h"


#define MAX_MESSAGE_LEN 100000
#define CHARS_PER_PAGE 10000 /* number of chars retrieved in each request */
#define MAX_FILE_NAME_LEN 1000
#define Z39_50_SERVICE "210"

#define WAISSEARCH_DATE "Thu Aug 27 1992"

#define MAX_KEYWORDS_LENGTH 1000
#define MAX_SERVER_LENGTH 1000
#define MAX_DATABASE_LENGTH 1000
#define MAX_SERVICE_LENGTH 1000

char* log_file_name;
FILE* logfile = NULL;

#define   error(x) 

record_struct *wais_interface (server_name, service, records, result, 
  keywords, database, maxhits)
   char *server_name;
   int service;
   record_struct *records;
   int *result;
   char *keywords;
   char *database;
   int maxhits;
{

char userInfo[500];
char* request_message = NULL; /* arbitrary message limit */
char* response_message = NULL; /* arbitrary message limit */
long request_buffer_length;  /* how of the request is left */
SearchResponseAPDU  *query_response;
SearchResponseAPDU  *retrieval_response;
WAISSearchResponse  *query_info, *retrieval_info;
long count, message_length = MAX_MESSAGE_LEN;
FILE *connection;
WAISSearchResponse *info = (WAISSearchResponse *) NULL;
int doc_number;
char tmp_str[1048576];

/*
if (keywords [strlen(keywords)-1] != ' ')
  strcat (keywords, " ");
*/
#if MAK
printf ("keywords |%s| on database %s\n", keywords, database);
#endif


if(((server_name == (char *) NULL) || (server_name[0] == '\0')) && service == 0)
  connection = NULL; /* do local searching */

request_message = (char*)s_malloc((size_t)message_length * sizeof(char));
response_message = (char*)s_malloc((size_t)message_length * sizeof(char));

sprintf(userInfo, "waissearch %s", VERSION);


if((message_length = init_connection(request_message, response_message,
  message_length, connection, userInfo)) <= 0) {
  *result = -1;
  return(records);
  }

request_buffer_length = message_length; /* how of the request is left */

if(generate_search_apdu(request_message + HEADER_LENGTH, 
  &request_buffer_length, keywords, database, NULL, maxhits) == NULL){
#if 0
      error(A_INTERR, "Error creating search APDU: request too large");
#endif
   }

if(interpret_message(request_message, message_length - request_buffer_length, 
  response_message, message_length, connection,false) == 0) {

      /* perhaps the server shut down on us, let's see: */

  if(connection != NULL) {
    fclose(connection);
    if((message_length = init_connection(request_message, response_message,
    message_length, connection, userInfo)) <= 0){
      *result = -1;
    return(records);
      }
    if(interpret_message(request_message, 
      message_length - request_buffer_length, response_message,
      message_length, connection, false /* true verbose */) == 0)
    fprintf(stderr,"Can't deliver message\n");
    }
 }

readSearchResponseAPDU(&query_response, response_message + HEADER_LENGTH);

query_info = (WAISSearchResponse *)query_response->DatabaseDiagnosticRecords;

if((info = (WAISSearchResponse *) query_response->DatabaseDiagnosticRecords) 
  != (WAISSearchResponse *) NULL){
  diagnosticRecord **d;
  d = info -> Diagnostics;
  if(d != NULL){
    long i;
     for(i = 0; d[i] != NULL; i++) {
       if(d[i]->ADDINFO != NULL) {
       sprintf(tmp_str, "Code: %s, %s;", d[i]->DIAG, d[i] ->ADDINFO);
         }
       }
    *result = -1;
    return(records);
    }
  }

if(!query_info -> DocHeaders) {
  *result = -1;
  return(records);
  }

if(query_info->DocHeaders[0]->BestMatch == -1) {
  *result = 0;
  return(records);
  }

for(doc_number = 0; doc_number < query_response -> NumberOfRecordsReturned;
   doc_number++){
 if(1){
   char *type;
   int i;
   WAISDocumentText *record;

/*
 * We are sending all information along with the link so it is a
 * "file" and cannot be expanded further
 */

   if(query_info->DocHeaders[doc_number]->Types == NULL)
      type = s_strdup("TEXT");
   else
      type = s_strdup(query_info->DocHeaders[doc_number]->Types[0]);

   request_buffer_length = message_length; /* how of the request is left */

   if(0 == generate_retrieval_apdu(request_message + HEADER_LENGTH,
     &request_buffer_length, query_info->DocHeaders[doc_number]->DocumentID, 
     CT_byte, 0, query_info->DocHeaders[doc_number]->DocumentLength,
     type, database)) {
     break;
     }
      
   if(0 == interpret_message(request_message, 
     message_length - request_buffer_length, response_message, message_length,
     connection, false /* true verbose */  
     )) { /* perhaps the server shut down on us, let's see: */

     if ( connection != NULL) {
       fclose(connection);
       if ((connection=connect_to_server(server_name,atoi(service))) == NULL) {
         fprintf (stderr, "Error opening connection to %s via service %d.\n",
         server_name, service);
         exit(-1);
         }
       if(0 == interpret_message(request_message, 
         message_length - request_buffer_length, response_message,
         message_length, connection, false /* true verbose */
         )) {
         printf("really couldn't deliver message");
         break;
       }
     }
   else {
     printf("Error deliverring message");
     break;
     }
   }

  readSearchResponseAPDU(&retrieval_response, response_message + HEADER_LENGTH);

  if(NULL == ((WAISSearchResponse *)
    retrieval_response->DatabaseDiagnosticRecords)->Text){
    }

  tmp_str[0] = '\0';

  record = ((WAISSearchResponse *)
    retrieval_response->DatabaseDiagnosticRecords)->Text[0];
      
  for(count = 0, i=0; count < record->DocumentText->size; count++){
    long ch = (unsigned char)record->DocumentText->bytes[count];
    if(27 == ch){

/* then we have an escape code */
/* if the next letter is '(' or ')', then ignore two letters */

      if('(' == record->DocumentText->bytes[count + 1] ||
        ')' == record->DocumentText->bytes[count + 1])
        count += 1;             /* it is a term marker */
      else
        count += 4;    /* it is a paragraph marker */
      }
    else
      tmp_str[i++] = ch;
      
    }

    tmp_str[i] = '\0';


    records = read_record_from_wais (records, tmp_str, NULL);

    freeWAISSearchResponse( retrieval_response->DatabaseDiagnosticRecords); 
    freeSearchResponseAPDU( retrieval_response);
    }
  else{
    }
  }

freeWAISSearchResponse(query_response->DatabaseDiagnosticRecords);         
freeSearchResponseAPDU( query_response);

s_free(request_message);
s_free(response_message);
   
return(records);
}
