/************************************************************************
RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and 
that no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
					scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
					markk@internic.net


Developed By: 
  Mark Kosters
***************************************************************************/

#include <stdio.h>
#include "defines.h"
#include "conf.h"
#include "misc.h"
#include "objects.h"
#include "xfer.h"
#include "client_msgs.h"
#ifdef USE_MKDB
#include "fileinfo.h"
#endif

/***************
  local prototypes
***************/
#ifdef USE_WAIS
static int open_wais_source();
#endif
#ifdef USE_MKDB
static int open_mkdb_source();
#endif

static int show_file_contents();
static void print_out_xfer();
static int valid_record();

/****************************************************************************
  displays the schema of the objects
****************************************************************************/
transfer_data (str)
char *str;

{
char filename[MAX_LINE];
char object_name[MAX_LINE], authority_area[MAX_LINE], serial_no[MAX_LINE];
char *cp, *next;
llist_struct *cur;
object_struct *object;
int ret = FALSE;
int foundobj = FALSE, foundrec=FALSE;

*object_name = '\0';
*authority_area = '\0';
*serial_no = '\0';
next = get_word (str, object_name);
if (next != NULL) { 
  cp = next;
  if ((next = get_word (cp, authority_area)) != NULL) 
    strcpy (serial_no, next);
  }
if (strcasecmp (object_name, "ALL") == 0)
  cur = return_all_defined_objects ("");
else
  cur = return_all_defined_objects(object_name);

while (cur != NULL) {
  foundobj = TRUE;
  object = cur->datum;
  if (object->type != ALL_OBJECTS) {
    if (ret)
       printf ("%%xfer\n");
#ifdef USE_WAIS
    sprintf (filename, "%s.src", object->local_db);
    ret = open_wais_source (object->name, filename, authority_area, serial_no);
#endif
#ifdef USE_MKDB
    sprintf (filename, "%s.db", object->local_db);
    ret = open_mkdb_source (object->name, filename, authority_area, serial_no);
    if (ret)
      foundrec = TRUE;
    }
#endif
  cur = cur->nxt;
  }
if (foundrec)
  print_ok();
else if (foundobj)
  print_error (NO_TRANSFER,"");
else
  print_error (NO_OBJECT_SCHEMA_FOUND,"");
set_passthru();
}

#ifdef USE_WAIS
/****************************************************************************
  opens the wais source file
****************************************************************************/
static int open_wais_source (object_name, filename, aa, soa)
char *object_name;
char *filename;
char *aa;
char *soa;

{
FILE *fp;
char line[MAX_LINE];
int in_description = FALSE;
int ret=0;

if ((fp = fopen (filename, "r")) == NULL) {
  print_error (MISC_DIAG, "Can not open data file '%s'", filename);
  return;
  }
while (fgets (line, MAX_LINE, fp) != NULL) {
  stripchar (line, '\n');
  Ltrim (line);
  if (!in_description && strstr (line, ":description") != NULL)
    in_description = TRUE;
  if (in_description && line[0] == '/') {
    if (ret)
      printf ("%%xfer\n");
    ret = show_file_contents (object_name, line, aa, soa);
    }
  }
return (ret);
}
#endif

#ifdef USE_MKDB
/****************************************************************************
  opens the source file
****************************************************************************/
static int open_mkdb_source (object_name, filename, aa, soa)
char *object_name;
char *filename;
char *aa;
char *soa;

{
FILE *fp;
char *fname;
int reset, ret=FALSE, retval=FALSE;

if (!read_fileinfo (filename)) {
  print_error (MISC_DIAG, "Can not open data file '%s'", filename);
  return;
  }
reset = 1;
while ((fname = get_fileinfo_data_files (reset)) != NULL) {
  reset = 0;
  if (ret) {
    printf ("%%xfer\n");
    retval = TRUE;
    }
  ret = show_file_contents (object_name, fname, aa, soa);
  }
clear_fileinfo();
return (retval);
}
#endif

/****************************************************************************
  displays the attributes of a particular object
****************************************************************************/
static int show_file_contents (object_name, filename, aa, soa)
char *object_name;
char *filename;
char *aa;
char *soa;

{
FILE *fp;
char buf[4096];
char line[MAX_LINE];
int ret=FALSE;
int addlines=TRUE;

if ((fp = fopen (filename, "r")) == NULL) {
  print_error (MISC_DIAG, "Can not open data file '%s'", filename);
  return;
  }
*buf= '\0';

while (fgets (line, MAX_LINE, fp) != NULL) {
  stripchar (line, '\n');
  if (record_separator (line)) {
    if (strlen(buf) && valid_record (buf, aa, soa)) {
      if (ret)
        printf ("%%xfer\n");
      print_out_xfer (object_name, buf);
      if (!addlines)
        print_error (MISC_DIAG, "record truncated");
      ret = TRUE;
      }
    *buf = '\0';
    }
  else if (addlines) {
    if (strlen (buf) + strlen (line) + 2 > 4096)
       addlines = FALSE;
    else {
      strcat (buf, "\n");
      strcat (buf, line);
      }
    }
  }
if (strlen(buf) && valid_record (buf, aa, soa)) {
  if (ret)
    printf ("%%xfer\n");
  print_out_xfer (object_name, buf);
  ret = TRUE;
  *buf = '\0';
  }
fclose (fp);
return (ret);
}

/****************************************************************************
  prints out the xfer stuff
****************************************************************************/
static void print_out_xfer (object_name, buf)
char *object_name;
char *buf;

{
int cnt=0;
char line[MAX_LINE], tag[MAX_LINE];
char object[MAX_LINE], attribute[MAX_LINE], datum[MAX_LINE];

set_attributes_list_from_object (FALSE, NULL, NULL);
while (delimstr(buf, "\n", line, cnt++)) {
  if (get_tuple(object, attribute, datum, line))
    printf ("%%xfer %s:%s:%s\n", object, attribute, datum);
  }
}

/****************************************************************************
  checks if valid record based on authority area and soa
  returns TRUE if is
          FALSE if not
****************************************************************************/
static int valid_record (buf, aa, soa)
char *buf;
char *aa;
char *soa;

{
int cnt=0;
char line[MAX_LINE], tag[MAX_LINE], datum[MAX_LINE];
attribute_struct *attrib = NULL;
int valid_aa = FALSE;
int valid_soa = FALSE;
attrib = NULL;

/** needs aa to go on **/
if (!strlen(aa))
  return (TRUE);
set_attributes_list_from_object (FALSE, NULL, NULL);
while (delimstr(buf, "\n", line, cnt++)) {
  if(parse_line(line, tag, datum)){
    attrib = set_attributes_list_from_object (TRUE, tag, datum);
    attrib = find_specific_attribute (attrib, tag);
    if (attrib != NULL && attrib->authority_area) {
      if (strcasecmp (datum, aa) == 0)
        valid_aa = TRUE;
      }
    else if (strlen (soa) && attrib != NULL && attrib->soa) {
      if (atoi (datum) > atoi (soa)) 
        valid_soa = TRUE;
      }
    else if (attrib == NULL && debug_status())
      printf ("xfer: bad line: |%s|\n", line);
    }
  }
if (valid_aa) {
  if (strlen(soa)) {
    if (valid_soa) 
      return (TRUE);
    else
      return (FALSE);
    }
  else 
    return (TRUE);
  }
else
  return (FALSE);
}
