/************************************************************************
RWhois Software

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and
that no charge is made for such copies.

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By:
  Mark Kosters
***************************************************************************/

/**************************************************************************
  Change history

9/13/94 Included additional col in build_unsorted_index to mention that 
        element deleted (TRUE) or not (FALSE).
***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "defines.h"
#include "fileinfo.h"
#include "conf.h"
#include "attributes.h"
#include "rwhois_supp.h"

/************
  local prototypes
*************/
static FILE *open_file();

/*************************************************************************
  main routine
**************************************************************************/
main(argc, argv)
int argc;
char *argv[];

{
extern int optind;
extern char *optarg;
char c;
FILE *rfp, *ifp;
char pwd[MAX_FILENAME];
char oldfname[MAX_FILENAME];
char newfname[MAX_FILENAME];
char *config_file= NULL;
char *default_dir= NULL;

/******
  command line parsing
******/

setbuf (stdout, NULL);
while ((c = getopt(argc, argv, "d:c:")) != EOF) {
  switch (c) {
    case 'c':
      config_file = optarg;
      break;
    case 'd':
      default_dir = optarg;
      break;
    }
  }

/******
  initialization
******/

getcwd (pwd, MAX_FILENAME);
if (argc == 1) {
  fprintf (stderr, "%s: usage <file>\n", argv[0]);
  exit (1);
  }

if (config_file == NULL)
  config_file = DEFAULT_RWHOIS_CONFIG_FILE;

if (!read_config_files(config_file, default_dir)) {
  printf ("problems initializing config enviro - exiting\n");
  exit (1);
  }

/********
  ok now do the real stuff
********/

chdir (pwd);

while (optind < argc) {
  printf ("converting '%s'\n", argv[optind]);
  strcpy (newfname, argv[optind]);
  strcpy (oldfname, argv[optind]);
  strcat (oldfname, ".old");
  rename (newfname, oldfname);
  if ((rfp = open_file (oldfname, "r")) == NULL ||
    (ifp = open_file (newfname, "w"))== NULL) {
    print_error (999, "error opening old and new files\n");
    exit(1);
    }
  convert_mkdb (rfp, ifp, newfname);
  fclose (rfp);
  fclose (ifp);
  optind++;
  }
}

/*************************************************************************
 opens the files
**************************************************************************/
static FILE *open_file (db_file, mode)
char *db_file;
char *mode;

{
FILE *fp;

if((fp = fopen(db_file, mode)) == (FILE *) NULL){
  print_error (999, "can not open %s mode %s", db_file, mode);
  return(NULL);
  }
return (fp);
}


/*************************************************************************
convert
**************************************************************************/
convert_mkdb (rfp, ifp, fname)
FILE *rfp, *ifp;
char *fname;

{
int line_no=0;
int num_recs = 0;
char line[MAX_LINE], datum[MAX_LINE];
attribute_struct *attrib;

reset_object_old();
while (fgets (line, MAX_LINE, rfp) != NULL) {
  if (!(++line_no%1000))
    fprintf (stderr, "%d lines\n", line_no);
  stripchar (line, '\n');
  if (record_separator(line)) { 
    num_recs++;
    reset_object();
    fprintf (ifp, "%s\n", line);
    }
  else if ((attrib = get_attribute_info_old (line, datum, fname, line_no)) !=
    NULL) {
    if (!attrib->def_object)
      fprintf (ifp, "%s:%s:%s\n", attrib->object_name, attrib->name, datum);
    }
  }
}
