/************************************************************************
RWhois Software

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and
that no charge is made for such copies.

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By:
  Mark Kosters
***************************************************************************/

/**************************************************************************
  Change history

9/13/94 Included additional col in build_unsorted_index to mention that 
        element deleted (TRUE) or not (FALSE).
***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "defines.h"
#include "fileinfo.h"
#include "conf.h"
#include "attributes.h"
#include "rwhois_supp.h"

#define USAGE_STR "%s: <-c rwhois_config_file> <file>\n"

#define DATA_FILE             "DATA"
#define INDEX_FILE            "INDEX"
/************
  local prototypes
*************/
static FILE *open_file();

/*************************************************************************
  main routine
**************************************************************************/
main(argc, argv)
int argc;
char *argv[];

{
extern int optind;
extern char *optarg;
char c;
int index_no;
int num_recs=0;
FILE *fp;
char pwd[MAX_FILENAME];
char *config_file = NULL;
char *default_dir = NULL;
char dbase_name[MAX_FILENAME];
char index_name[MAX_FILENAME];
char unsorted_name[MAX_FILENAME];
char rwhois_area[MAX_FILENAME];

/******
  initialization
******/

*dbase_name = '\0';
*index_name = '\0';
*rwhois_area = '\0';
getcwd (pwd, MAX_FILENAME); /* thanks to A.Linton@aarnet.edu.au */

/******
  command line parsing
*******/

setbuf (stdout, NULL);
while ((c = getopt(argc, argv, "d:n:c:")) != EOF) {
  switch (c) {
    case 'c':
      config_file = optarg;
      break;
    }
  }
if (argc == 1) {
  fprintf (stderr, USAGE_STR, argv[0]);
  exit (1);
  }

/********
  read in rwhois configuration items
********/

if (config_file == NULL) {
  default_dir = DEFAULT_RWHOIS_DATA_DIRECTORY;
  set_default_dir (default_dir);
  }
if (config_file == NULL)
  config_file = DEFAULT_RWHOIS_CONFIG_FILE;

if (!read_config_files(config_file, default_dir))  
  /* do 1-time initialization */
  exit (1);

/*******
  change back to current directory and index
*******/

chdir (pwd);

while (optind<argc) {
  printf ("rupdate: have |%s|\n", argv[optind]);
  optind++;
  } 
}

/*************************************************************************
 opens the files
**************************************************************************/
static FILE *open_file (db_file, mode)
char *db_file;
char *mode;

{
FILE *fp;

if((fp = fopen(db_file, mode)) == (FILE *) NULL){
  print_error (999, "can not open %s mode %s", db_file, mode);
  return(NULL);
  }
return (fp);
}
